@@ -9,17 +9,80 @@ import copy
 from astropy.io import fits
 from .utils import contiguous_regions, jit, HAS_NUMBA
 from .utils import assign_value_if_none, apply_function_if_none
+from .utils import check_iterables_close
 from stingray.exceptions import StingrayError
 
+
 __all__ = ['load_gtis', 'check_gtis',
            'create_gti_mask_jit', 'create_gti_mask',
            'create_gti_mask_complete', 'create_gti_from_condition',
            'cross_two_gtis', 'cross_gtis', 'get_btis',
            'get_gti_extensions_from_pattern', 'get_gti_from_all_extensions',
-           'get_gti_from_hdu', 'gti_len',
+           'get_gti_from_hdu', 'get_gti_lengths', 'get_total_gti_length',
            'check_separate', 'append_gtis', 'join_gtis',
+           'generate_indices_of_gti_boundaries',
            'time_intervals_from_gtis', 'bin_intervals_from_gtis',
-           'gti_border_bins']
+           'gti_border_bins',
+           'generate_indices_of_segment_boundaries_unbinned',
+           'generate_indices_of_segment_boundaries_binned']
+
+
+def gti_len(gti):
+    """Deprecated, will be removed in version 2.0. Use get_total_gti_length."""
+    warnings.warn("This function is deprecated. Use get_total_gti_length "\
+                  "instead", DeprecationWarning)
+    return get_total_gti_length(gti, minlen=0)
+
+
+def get_gti_lengths(gti):
+    """
+    Calculate the length of each Good Time Interval.
+
+    Parameters
+    ----------
+    gti : [[gti00, gti01], [gti10, gti11], ...]
+        The list of good time intervals.
+
+    Returns
+    -------
+    lengths : `np.ndarray`
+        List of GTI lengths.
+
+    Examples
+    --------
+    >>> gti = [[0, 1000], [1000, 1001], [3000, 3020]]
+    >>> np.allclose(get_gti_lengths(gti), [1000, 1, 20])
+    True
+    """
+    return np.diff(gti, axis=1).flatten()
+
+
+def get_total_gti_length(gti, minlen=0):
+    """
+    Calculate the total exposure during Good Time Intervals.
+
+    Parameters
+    ----------
+    gti : [[gti00, gti01], [gti10, gti11], ...]
+        The list of good time intervals.
+    minlen : float
+        Minimum GTI length to consider.
+
+    Returns
+    -------
+    length : float
+        The total exposure during GTIs.
+
+    Examples
+    --------
+    >>> gti = [[0, 1000], [1000, 1001], [3000, 3020]]
+    >>> get_total_gti_length(gti)
+    1021
+    >>> get_total_gti_length(gti, minlen=5)
+    1020
+    """
+    lengths = get_gti_lengths(gti)
+    return np.sum(lengths[lengths >= minlen])
 
 
 def load_gtis(fits_file, gtistring=None):
@@ -30,18 +93,16 @@ def load_gtis(fits_file, gtistring=None):
     Parameters
     ----------
     fits_file : str
-        File name and path for the fits file with the GTIs to be loaded
+        File name and path for the FITS file with the GTIs to be loaded.
 
     gtistring : str
-        If the name of the extension with the GTIs is not ``GTI``, the alternative
-        name can be set with this parameter
+        If the name of the FITS extension with the GTIs is not ``GTI``, the
+        alternative name can be set with this parameter.
 
     Returns
     -------
     gti_list : list
         A list of GTI ``(start, stop)`` pairs extracted from the FITS file.
-
-
     """
 
     gtistring = assign_value_if_none(gtistring, 'GTI')
@@ -59,19 +120,20 @@ def load_gtis(fits_file, gtistring=None):
 
 
 def get_gti_extensions_from_pattern(lchdulist, name_pattern="GTI"):
-    """Gets the GTI extensions that match a given pattern.
+    """
+    Gets the GTI extensions that match a given pattern.
 
     Parameters
     ----------
     lchdulist: `:class:astropy.io.fits.HDUList` object
         The full content of a FITS file.
     name_pattern: str
-        Pattern indicating all the GTI extensions
+        Pattern indicating all the GTI extensions.
 
     Returns
     -------
     ext_list: list
-        List of GTI extension numbers whose name matches the input pattern
+        List of GTI extension numbers whose name matches the input pattern.
 
     Examples
     --------
@@ -97,18 +159,42 @@ def get_gti_extensions_from_pattern(lchdulist, name_pattern="GTI"):
     return gtiextn
 
 
+def hdu_contains_gti(hdu):
+    """
+    Test if a given FITS HDU contains a list of GTIs.
+
+    Examples
+    --------
+    >>> from astropy.io import fits
+    >>> start = np.arange(0, 300, 100)
+    >>> stop = start + 50.
+    >>> s1 = fits.Column(name='START', array=start, format='D')
+    >>> s2 = fits.Column(name='STOP', array=stop, format='D')
+    >>> hdu1 = fits.TableHDU.from_columns([s1, s2], name='BLABLA')
+    >>> hdu_contains_gti(hdu1)
+    True
+    >>> s2 = fits.Column(name='blabla', array=stop, format='D')
+    >>> hdu1 = fits.TableHDU.from_columns([s1, s2], name='BLABLA')
+    >>> hdu_contains_gti(hdu1)
+    False
+    """
+    colnames = [c.lower() for c in hdu.data.columns.names]
+    return "start" in colnames and "stop" in colnames
+
+
 def get_gti_from_hdu(gtihdu):
-    """Get the GTIs from a given extension.
+    """
+    Get the GTIs from a given FITS extension.
 
     Parameters
     ----------
     gtihdu: `:class:astropy.io.fits.TableHDU` object
-        The GTI hdu
+        The GTI HDU.
 
     Returns
     -------
     gti_list: [[gti00, gti01], [gti10, gti11], ...]
-        List of good time intervals
+        List of good time intervals.
 
     Examples
     --------
@@ -138,10 +224,10 @@ def get_gti_from_hdu(gtihdu):
     return gti_list
 
 
-def get_gti_from_all_extensions(
-    lchdulist, accepted_gtistrings=["GTI"], det_numbers=None
-):
-    """Intersect the GTIs from the all accepted extensions.
+def get_gti_from_all_extensions(lchdulist, accepted_gtistrings=["GTI"],
+                                det_numbers=None):
+    """
+    Intersect the GTIs from the all accepted extensions.
 
     Parameters
     ----------
@@ -198,7 +284,8 @@ def get_gti_from_all_extensions(
 
 
 def check_gtis(gti):
-    """Check if GTIs are well-behaved.
+    """
+    Check if GTIs are well-behaved.
 
     Check that:
 
@@ -219,13 +306,13 @@ def check_gtis(gti):
         If GTIs have overlapping or displaced values
     """
     if len(gti) < 1:
-        raise ValueError("Empty GTIs")
+        raise ValueError("Empty GTIs.")
 
     for g in gti:
         if np.size(g) != 2 or np.ndim(g) != 1:
             raise TypeError(
-                "Please check the formatting of GTIs. They need to be"
-                " provided as [[gti00, gti01], [gti10, gti11], ...]")
+                "Please check the formatting of the GTIs. They need to be"
+                " provided as [[gti00, gti01], [gti10, gti11], ...].")
 
     gti = np.array(gti)
     gti_start = gti[:, 0]
@@ -233,20 +320,21 @@ def check_gtis(gti):
 
     # Check that GTIs are well-behaved
     if not np.all(gti_end >= gti_start):
-        raise ValueError('This GTI end times must be larger than '
-                         'GTI start times')
+        raise ValueError('The GTI end times must be larger than the '
+                         'GTI start times.')
 
     # Check that there are no overlaps in GTIs
     if not np.all(gti_start[1:] >= gti_end[:-1]):
-        raise ValueError('This GTI has overlaps')
+        raise ValueError('This GTI has overlaps.')
 
     return
 
 
 @jit(nopython=True)
-def create_gti_mask_jit(time, gtis, mask, gti_mask, min_length=0):  # pragma: no cover
+def create_gti_mask_jit(time, gtis, mask, gti_mask,
+                        min_length=0):  # pragma: no cover
     """
-    Compiled and fast function to create gti mask.
+    Compiled and fast function to create GTI mask.
 
     Parameters
     ----------
@@ -254,18 +342,19 @@ def create_gti_mask_jit(time, gtis, mask, gti_mask, min_length=0):  # pragma: no
         An array of time stamps
 
     gtis : iterable of ``(start, stop)`` pairs
-        The list of GTIs
+        The list of GTIs.
 
     mask : numpy.ndarray
         A pre-assigned array of zeros of the same shape as ``time``
         Records whether a time stamp is part of the GTIs.
 
     gti_mask : numpy.ndarray
-        A pre-assigned array zeros in the same shape as ``time``; records start/stop of GTIs
+        A pre-assigned array zeros in the same shape as ``time``; records
+        start/stop of GTIs.
 
     min_length : float
-        An optional minimum length for the GTIs to be applied. Only GTIs longer than ``min_length`` will
-        be considered when creating the mask.
+        An optional minimum length for the GTIs to be applied. Only GTIs longer
+        than ``min_length`` will be considered when creating the mask.
 
     """
     gti_el = -1
@@ -296,7 +385,8 @@ def create_gti_mask_jit(time, gtis, mask, gti_mask, min_length=0):  # pragma: no
 
 def create_gti_mask(time, gtis, safe_interval=None, min_length=0,
                     return_new_gtis=False, dt=None, epsilon=0.001):
-    """Create GTI mask.
+    """
+    Create GTI mask.
 
     Assumes that no overlaps are present between GTIs
 
@@ -324,18 +414,19 @@ def create_gti_mask(time, gtis, safe_interval=None, min_length=0,
         If ``True```, return the list of new GTIs (if ``min_length > 0``)
 
     dt : float
-        Time resolution of the data, i.e. the interval between time stamps
+        Time resolution of the data, i.e. the interval between time stamps.
 
     epsilon : float
-        fraction of ``dt`` that is tolerated at the borders of a GTI
+        Fraction of ``dt`` that is tolerated at the borders of a GTI.
 
     Returns
     -------
     mask : bool array
-        A mask labelling all time stamps that are included in the GTIs versus those that are not.
+        A mask labelling all time stamps that are included in the GTIs versus
+        those that are not.
 
     new_gtis : ``Nx2`` array
-        An array of new GTIs created by this function
+        An array of new GTIs created by this function.
     """
     gtis = np.array(gtis, dtype=np.longdouble)
     if time.size == 0:
@@ -389,8 +480,8 @@ def create_gti_mask(time, gtis, safe_interval=None, min_length=0,
                             (gtis_to_mask - time[0]).astype(np.float64),
                             mask, gti_mask=gti_mask,
                             min_length=float(
-                                    min_length - 2 * (1 + epsilon) * dt
-                            ))
+            min_length - 2 * (1 + epsilon) * dt
+        ))
 
     if return_new_gtis:
         return mask, gtis_new[gtimask]
@@ -400,18 +491,18 @@ def create_gti_mask(time, gtis, safe_interval=None, min_length=0,
 
 def create_gti_mask_complete(time, gtis, safe_interval=0, min_length=0,
                              return_new_gtis=False, dt=None, epsilon=0.001):
+    """
+    Create GTI mask, allowing for non-constant ``dt``.
 
-    """Create GTI mask, allowing for non-constant ``dt``.
-
-    Assumes that no overlaps are present between GTIs
+    Assumes that no overlaps are present between GTIs.
 
     Parameters
     ----------
     time : numpy.ndarray
-        An array of time stamps
+        An array of time stamps.
 
     gtis : ``[[g0_0, g0_1], [g1_0, g1_1], ...]``, float array-like
-        The list of GTIs
+        The list of GTIs.
 
 
     Other parameters
@@ -421,25 +512,26 @@ def create_gti_mask_complete(time, gtis, safe_interval=0, min_length=0,
         and the end (if pair of values) of GTIs.
 
     min_length : float
-        An optional minimum length for the GTIs to be applied. Only GTIs longer than ``min_length`` will
-        be considered when creating the mask.
+        An optional minimum length for the GTIs to be applied. Only GTIs longer
+        than ``min_length`` will be considered when creating the mask.
 
     return_new_gtis : bool
-        If ``True``, return the list of new GTIs (if ``min_length > 0``)
+        If ``True``, return the list of new GTIs (if ``min_length > 0``).
 
     dt : float
-        Time resolution of the data, i.e. the interval between time stamps
+        Time resolution of the data, i.e. the interval between time stamps.
 
     epsilon : float
-        fraction of ``dt`` that is tolerated at the borders of a GTI
+        Fraction of ``dt`` that is tolerated at the borders of a GTI.
 
     Returns
     -------
     mask : bool array
-        A mask labelling all time stamps that are included in the GTIs versus those that are not.
+        A mask labelling all time stamps that are included in the GTIs versus
+        those that are not.
 
     new_gtis : Nx2 array
-        An array of new GTIs created by this function
+        An array of new GTIs created by this function.
     """
 
     check_gtis(gtis)
@@ -481,12 +573,13 @@ def create_gti_mask_complete(time, gtis, safe_interval=0, min_length=0,
 
 def create_gti_from_condition(time, condition,
                               safe_interval=0, dt=None):
-    """Create a GTI list from a time array and a boolean mask (``condition``).
+    """
+    Create a GTI list from a time array and a boolean mask (``condition``).
 
     Parameters
     ----------
     time : array-like
-        Array containing time stamps
+        Array containing time stamps.
 
     condition : array-like
         An array of bools, of the same length of time.
@@ -495,7 +588,7 @@ def create_gti_from_condition(time, condition,
     Returns
     -------
     gtis : ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
-        The newly created GTIs
+        The newly created GTIs.
 
     Other parameters
     ----------------
@@ -533,7 +626,8 @@ def create_gti_from_condition(time, condition,
 
 
 def cross_two_gtis(gti0, gti1):
-    """Extract the common intervals from two GTI lists *EXACTLY*.
+    """
+    Extract the common intervals from two GTI lists *EXACTLY*.
 
     Parameters
     ----------
@@ -544,7 +638,7 @@ def cross_two_gtis(gti0, gti1):
     Returns
     -------
     gtis : ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
-        The newly created GTIs
+        The newly created GTIs.
 
     See Also
     --------
@@ -639,13 +733,14 @@ def cross_gtis(gti_list):
     Parameters
     ----------
     gti_list : array-like
-        List of GTI arrays, each one in the usual format ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTI arrays, each one in the usual format
+        ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
     Returns
     -------
     gti0: 2-d float array
         ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
-        The newly created GTIs
+        The newly created GTIs.
 
     See Also
     --------
@@ -690,20 +785,20 @@ def get_btis(gtis, start_time=None, stop_time=None):
     Parameters
     ----------
     gtis : iterable
-        A list of GTIs
+        A list of GTIs.
 
     start_time : float
-        Optional start time of the overall observation (e.g. can be earlier than the first time stamp
-        in ``gtis``.
+        Optional start time of the overall observation (e.g. can be earlier
+        than the first time stamp in ``gtis``).
 
     stop_time : float
-        Optional stop time of the overall observation (e.g. can be later than the last time stamp in
-        ``gtis``.
+        Optional stop time of the overall observation (e.g. can be later than
+        the last time stamp in``gtis``).
 
     Returns
     -------
     btis : numpy.ndarray
-        A list of bad time intervals
+        A list of bad time intervals.
     """
     # Check GTIs
     if len(gtis) == 0:
@@ -732,25 +827,6 @@ def get_btis(gtis, start_time=None, stop_time=None):
     return np.asarray(btis)
 
 
-def gti_len(gti):
-    """
-    Return the total good time from a list of GTIs.
-
-    Parameters
-    ----------
-    gti : iterable
-        A list of Good Time Intervals
-
-    Returns
-    -------
-    gti_len : float
-        The sum of lengths of all GTIs
-
-    """
-    gti = np.array(gti)
-    return np.sum(gti[:, 1] - gti[:, 0])
-
-
 @jit(nopython=True)
 def _check_separate(gti0, gti1):
     """Numba-compiled core of ``check_separate``."""
@@ -780,15 +856,15 @@ def check_separate(gti0, gti1):
     Parameters
     ----------
     gti0: 2-d float array
-        List of GTIs of form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTIs of form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
     gti1: 2-d float array
-        List of GTIs of form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTIs of form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
     Returns
     -------
     separate: bool
-        ``True`` if GTIs are mutually exclusive, ``False`` if not
+        ``True`` if GTIs are mutually exclusive, ``False`` if not.
 
     Examples
     --------
@@ -832,10 +908,10 @@ def check_separate(gti0, gti1):
 
 
 def join_equal_gti_boundaries(gti):
-    """If the start of a GTI is right at the end of another, join them.
-
     """
-    new_gtis=[]
+    If the start of a GTI is right at the end of another, join them.
+    """
+    new_gtis = []
     for l in gti:
         new_gtis.append(l)
     touching = gti[:-1, 1] == gti[1:, 0]
@@ -852,7 +928,8 @@ def join_equal_gti_boundaries(gti):
 
 
 def append_gtis(gti0, gti1):
-    """Union of two non-overlapping GTIs.
+    """
+    Union of two non-overlapping GTIs.
 
     If the two GTIs "touch", this is tolerated and the touching GTIs are
     joined in a single one.
@@ -860,15 +937,15 @@ def append_gtis(gti0, gti1):
     Parameters
     ----------
     gti0: 2-d float array
-        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
     gti1: 2-d float array
-        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
     Returns
     -------
     gti: 2-d float array
-        The newly created GTI
+        The newly created GTI array.
 
     Examples
     --------
@@ -898,7 +975,8 @@ def append_gtis(gti0, gti1):
 
 
 def join_gtis(gti0, gti1):
-    """Union of two GTIs.
+    """
+    Union of two GTIs.
 
     If GTIs are mutually exclusive, it calls ``append_gtis``. Otherwise we put
     the extremes of partially overlapping GTIs on an ideal line and look at the
@@ -974,9 +1052,10 @@ def join_gtis(gti0, gti1):
     return np.sort(final_gti, axis=0)
 
 
-def time_intervals_from_gtis(gtis, chunk_length, fraction_step=1,
+def time_intervals_from_gtis(gtis, segment_size, fraction_step=1,
                              epsilon=1e-5):
-    """Compute start/stop times of equal time intervals, compatible with GTIs.
+    """
+    Compute start/stop times of equal time intervals, compatible with GTIs.
 
     Used to start each FFT/PDS/cospectrum from the start of a GTI,
     and stop before the next gap in data (end of GTI).
@@ -986,13 +1065,14 @@ def time_intervals_from_gtis(gtis, chunk_length, fraction_step=1,
     gtis : 2-d float array
         List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
 
-    chunk_length : float
+    segment_size : float
         Length of the time segments
 
     fraction_step : float
-        If the step is not a full ``chunk_length`` but less (e.g. a moving window),
-        this indicates the ratio between step step and ``chunk_length`` (e.g.
-        0.5 means that the window shifts of half ``chunk_length``)
+        If the step is not a full ``segment_size`` but less (e.g. a moving
+        window), this indicates the ratio between step step and
+        ``segment_size`` (e.g. ``0.5`` means that the window shifts by half
+        ``segment_size``).
 
     Returns
     -------
@@ -1005,42 +1085,95 @@ def time_intervals_from_gtis(gtis, chunk_length, fraction_step=1,
     """
     spectrum_start_times = np.array([], dtype=np.longdouble)
     for g in gtis:
-        if g[1] - g[0] + epsilon < chunk_length:
+        if g[1] - g[0] + epsilon < segment_size:
             continue
 
-        newtimes = np.arange(g[0], g[1] - chunk_length + epsilon,
-                             np.longdouble(chunk_length) * fraction_step,
+        newtimes = np.arange(g[0], g[1] - segment_size + epsilon,
+                             np.longdouble(segment_size) * fraction_step,
                              dtype=np.longdouble)
         spectrum_start_times = \
             np.append(spectrum_start_times,
                       newtimes)
 
     assert len(spectrum_start_times) > 0, \
-        ("No GTIs are equal to or longer than chunk_length.")
-    return spectrum_start_times, spectrum_start_times + chunk_length
+        ("No GTIs are equal to or longer than segment_size.")
+    return spectrum_start_times, spectrum_start_times + segment_size
 
 
-def bin_intervals_from_gtis(gtis, chunk_length, time, dt=None, fraction_step=1,
+def calculate_segment_bin_start(startbin, stopbin, nbin, fraction_step=1):
+    """Get the starting indices of intervals of equal length.
+
+    A bit like `np.arange`, but checks that the last number is
+    at least ``nbin`` less than ``stopbin``. Useful when getting
+    starting intervals of equal chunks of a binned light curve.
+
+    It is possible to make these intervals sliding, through the
+    ``fraction_step`` parameter.
+
+    Parameters
+    ----------
+    startbin : int
+        Starting bin of the interval.
+
+    stopbin : int
+        Last bin of the interval.
+
+    nbin : int
+        Number of bins in each interval.
+
+    Other Parameters
+    ----------------
+    fraction_step : float
+        If the step is not a full ``nbin`` but less (e.g. a moving window),
+        this indicates the ratio between the step and ``nbin`` (e.g.
+        ``0.5`` means that the window shifts by half ``nbin``).
+
+    Returns
+    -------
+    spectrum_start_bins : array-like
+        List of starting bins in the original time array to use in spectral
+        calculations.
+
+    Examples
+    --------
+    >>> st = calculate_segment_bin_start(0, 10000, 10000)
+    >>> st[-1]
+    0
+    >>> st = calculate_segment_bin_start(0, 5, 2)
+    >>> st[-1]
+    2
+    >>> st = calculate_segment_bin_start(0, 6, 2)
+    >>> st[-1]
+    4
+    """
+    st = np.arange(startbin, stopbin, int(nbin * fraction_step), dtype=int)
+    if st[-1] + nbin > stopbin:
+        return st[:-1]
+    return st
+
+
+def bin_intervals_from_gtis(gtis, segment_size, time, dt=None, fraction_step=1,
                             epsilon=0.001):
-    """Compute start/stop times of equal time intervals, compatible with GTIs, and map them
-    to the indices of an array of time stamps.
+    """
+    Compute start/stop times of equal time intervals, compatible with GTIs,
+    and map them to the indices of an array of time stamps.
 
     Used to start each FFT/PDS/cospectrum from the start of a GTI,
     and stop before the next gap in data (end of GTI).
     In this case, it is necessary to specify the time array containing the
     times of the light curve bins.
-    Returns start and stop bins of the intervals to use for the PDS
+    Returns start and stop bins of the intervals to use for the PDS.
 
     Parameters
     ----------
     gtis : 2-d float array
-        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
-    chunk_length : float
-        Length of each time segment
+    segment_size : float
+        Length of each time segment.
 
     time : array-like
-        Array of time stamps
+        Array of time stamps.
 
     Other Parameters
     ----------------
@@ -1048,12 +1181,14 @@ def bin_intervals_from_gtis(gtis, chunk_length, time, dt=None, fraction_step=1,
         Time resolution of the light curve.
 
     epsilon : float, default 0.001
-        The tolerance, in fraction of ``dt``, for the comparisons at the borders
+        The tolerance, in fraction of ``dt``, for the comparisons at the
+        borders.
 
     fraction_step : float
-        If the step is not a full ``chunk_length`` but less (e.g. a moving window),
-        this indicates the ratio between step step and ``chunk_length`` (e.g.
-        ``0.5`` means that the window shifts of half ``chunk_length``)
+        If the step is not a full ``segment_size`` but less (e.g. a moving
+        window), this indicates the ratio between step step and
+        ``segment_size`` (e.g. ``0.5`` means that the window shifts by half
+        ``segment_size``).
 
     Returns
     -------
@@ -1068,11 +1203,11 @@ def bin_intervals_from_gtis(gtis, chunk_length, time, dt=None, fraction_step=1,
     --------
     >>> time = np.arange(0.5, 13.5)
 
-    >>> gtis = [[0, 5], [6, 8]]
+    >>> gtis = [[0, 5], [6, 8], [9, 10]]
 
-    >>> chunk_length = 2
+    >>> segment_size = 2
 
-    >>> start_bins, stop_bins = bin_intervals_from_gtis(gtis,chunk_length,time)
+    >>> start_bins, stop_bins = bin_intervals_from_gtis(gtis,segment_size,time)
 
     >>> np.allclose(start_bins, [0, 2, 6])
     True
@@ -1083,61 +1218,75 @@ def bin_intervals_from_gtis(gtis, chunk_length, time, dt=None, fraction_step=1,
     >>> np.allclose(time[start_bins[1]:stop_bins[1]], [2.5, 3.5])
     True
     """
+    time = np.asarray(time)
+    gtis = np.asarray(gtis)
     if dt is None:
         dt = np.median(np.diff(time))
 
     epsilon_times_dt = epsilon * dt
-    nbin = int(chunk_length / dt)
+    nbin = np.rint(segment_size / dt).astype(int)
 
     if time[-1] < np.min(gtis) or time[0] > np.max(gtis):
         raise ValueError("Invalid time interval for the given GTIs")
 
     spectrum_start_bins = np.array([], dtype=int)
-    time_low = time - dt / 2
-    time_high = time + dt / 2
-    for g in gtis:
-        if (g[1] - g[0] + epsilon_times_dt) < chunk_length:
-            continue
-        good_low = time_low >= (g[0] - epsilon_times_dt)
-        good_up = time_high <= (g[1] + epsilon_times_dt)
-        good = good_low & good_up
-        t_good = time[good]
-        if len(t_good) == 0:
-            continue
-        startbin = np.argmin(np.abs(time - dt / 2 - g[0]))
-        stopbin = np.searchsorted(time + dt / 2, g[1], 'right') + 1
-        if stopbin > len(time):
-            stopbin = len(time)
 
-        if time[startbin] < (g[0] + dt / 2 - epsilon_times_dt):
+    gti_low = gtis[:, 0] + dt / 2 - epsilon_times_dt
+    gti_up = gtis[:, 1] - dt / 2 + epsilon_times_dt
+
+    for g0, g1 in zip(gti_low, gti_up):
+        if (g1 - g0 + dt + epsilon_times_dt) < segment_size:
+            continue
+        startbin, stopbin = np.searchsorted(time, [g0, g1], "left")
+        stopbin += 1
+        if stopbin > time.size:
+            stopbin = time.size
+
+        if time[startbin] < g0:
             startbin += 1
         # Would be g[1] - dt/2, but stopbin is the end of an interval
         # so one has to add one bin
-        if time[stopbin - 1] > (g[1] - dt / 2 + epsilon_times_dt):
+        if time[stopbin - 1] > g1:
             stopbin -= 1
 
-        newbins = np.arange(startbin, stopbin - nbin + 1,
-                            int(nbin * fraction_step), dtype=int)
+        newbins = calculate_segment_bin_start(
+            startbin, stopbin, nbin, fraction_step=fraction_step)
         spectrum_start_bins = \
             np.append(spectrum_start_bins,
                       newbins)
     assert len(spectrum_start_bins) > 0, \
-        ("No GTIs are equal to or longer than chunk_length.")
+        ("No GTIs are equal to or longer than segment_size.")
     return spectrum_start_bins, spectrum_start_bins + nbin
 
 
 def gti_border_bins(gtis, time, dt=None, epsilon=0.001):
-    """Find the indices in a time array corresponding to the borders of GTIs.
+    """
+    Find the indices in a time array corresponding to the borders of GTIs.
 
     GTIs shorter than the bin time are not returned.
 
     Parameters
     ----------
     gtis : 2-d float array
-        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``
+        List of GTIs of the form ``[[gti0_0, gti0_1], [gti1_0, gti1_1], ...]``.
 
     time : array-like
-        Time stamps of light curve bins
+        Array of time stamps.
+
+    Other Parameters
+    ----------------
+    dt : float, default median(diff(time))
+        Time resolution of the light curve.
+
+    epsilon : float, default 0.001
+        The tolerance, in fraction of ``dt``, for the comparisons at the
+        borders.
+
+    fraction_step : float
+        If the step is not a full ``segment_size`` but less (e.g. a moving
+        window), this indicates the ratio between step step and
+        ``segment_size`` (e.g. ``0.5`` means that the window shifts by half
+        ``segment_size``).
 
     Returns
     -------
@@ -1152,6 +1301,11 @@ def gti_border_bins(gtis, time, dt=None, epsilon=0.001):
     --------
     >>> times = np.arange(0.5, 13.5)
 
+    >>> gti_border_bins([[16., 18.]], times)
+    Traceback (most recent call last):
+        ...
+    ValueError: Invalid time interval for the given GTIs
+
     >>> start_bins, stop_bins = gti_border_bins(
     ...    [[0, 5], [6, 8]], times)
 
@@ -1159,44 +1313,247 @@ def gti_border_bins(gtis, time, dt=None, epsilon=0.001):
     True
     >>> np.allclose(stop_bins, [5, 8])
     True
-    >>> np.allclose(times[start_bins[0]:stop_bins[0]], [ 0.5, 1.5, 2.5, 3.5, 4.5])
+    >>> np.allclose(times[start_bins[0]:stop_bins[0]], [0.5, 1.5, 2.5, 3.5, 4.5])
     True
     >>> np.allclose(times[start_bins[1]:stop_bins[1]], [6.5, 7.5])
     True
     """
+    time = np.asarray(time)
+    gtis = np.asarray(gtis)
     if dt is None:
         dt = np.median(np.diff(time))
 
     epsilon_times_dt = epsilon * dt
 
+    if time[-1] < np.min(gtis) or time[0] > np.max(gtis):
+        raise ValueError("Invalid time interval for the given GTIs")
+
     spectrum_start_bins = []
     spectrum_stop_bins = []
 
-    for g in gtis:
-        # Epsilons here resolve a numerical problem when time edges end up
-        # outside GTIs.
-        good = ((time - dt / 2 >= g[0] - epsilon_times_dt) &
-                (time + dt / 2 <= g[1] + epsilon_times_dt))
-        t_good = time[good]
-        if len(t_good) == 0:
-            continue
-        startbin = np.argmin(np.abs(time - dt / 2 - g[0]))
-        stopbin = np.searchsorted(time + dt / 2, g[1], 'right') + 1
-        if stopbin > len(time):
-            stopbin = len(time)
+    gti_low = gtis[:, 0] + dt / 2 - epsilon_times_dt
+    gti_up = gtis[:, 1] - dt / 2 + epsilon_times_dt
 
-        if time[startbin] < (g[0] + dt / 2 - epsilon_times_dt):
+    for g0, g1 in zip(gti_low, gti_up):
+        startbin, stopbin = np.searchsorted(time, [g0, g1], "left")
+        stopbin += 1
+        if stopbin > time.size:
+            stopbin = time.size
+
+        if time[startbin] < g0:
             startbin += 1
         # Would be g[1] - dt/2, but stopbin is the end of an interval
         # so one has to add one bin
-        if time[stopbin - 1] > (g[1] - dt / 2 + epsilon_times_dt):
+        if time[stopbin - 1] > g1:
             stopbin -= 1
-        spectrum_start_bins.append([startbin])
-        spectrum_stop_bins.append([stopbin])
 
-    spectrum_start_bins = np.concatenate(spectrum_start_bins)
-    spectrum_stop_bins = np.concatenate(spectrum_stop_bins)
+        spectrum_start_bins.append(startbin)
+        spectrum_stop_bins.append(stopbin)
 
-    assert len(spectrum_start_bins) > 0, \
-        ("No GTIs are equal to or longer than chunk_length.")
-    return spectrum_start_bins, spectrum_stop_bins
+    return np.array(spectrum_start_bins), np.array(spectrum_stop_bins)
+
+
+def generate_indices_of_boundaries(times, gti, segment_size=None, dt=0):
+    """
+    Get index boundaries and times from different parts of the observation.
+
+    It wraps around `generate_indices_of_gti_boundaries`,
+    `generate_indices_of_segment_boundaries_binned`, and
+    `generate_indices_of_segment_boundaries_unbinned` depending on:
+
+    + ``segment_size`` being ``None`` (give GTI boundaries, segment boundaries
+      otherwise)
+    + ``dt`` being 0 or nonzero (unevenly sampled, evenly sampled otherwise)
+
+    Examples
+    --------
+    >>> times = [0.1, 0.2, 0.5, 0.8, 1.1]
+    >>> gtis = [[0, 0.55], [0.6, 2.1]]
+    >>> vals0 = generate_indices_of_boundaries(times, gtis, segment_size=None)
+    >>> vals1 = generate_indices_of_gti_boundaries(times, gtis)
+    >>> check_iterables_close(vals0, vals1)
+    True
+    >>> vals0 = generate_indices_of_boundaries(times, gtis, segment_size=0.5)
+    >>> vals1 = generate_indices_of_segment_boundaries_unbinned(times, gtis, segment_size=0.5)
+    >>> check_iterables_close(vals0, vals1)
+    True
+    >>> times = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7]
+    >>> gtis = [[0.05, 0.55]]
+    >>> vals0 = generate_indices_of_boundaries(times, gtis, segment_size=0.5, dt=0.1)
+    >>> vals1 = generate_indices_of_segment_boundaries_binned(times, gtis, 0.5, dt=0.1)
+    >>> check_iterables_close(vals0, vals1)
+    True
+    """
+    if segment_size is not None:
+        if dt is None or dt == 0:
+            segment_iter = generate_indices_of_segment_boundaries_unbinned(
+                times, gti, segment_size)
+        else:
+            segment_iter = generate_indices_of_segment_boundaries_binned(
+                times, gti, segment_size, dt=dt)
+    else:
+        segment_iter = generate_indices_of_gti_boundaries(times, gti, dt=0)
+    return segment_iter
+
+
+def generate_indices_of_gti_boundaries(times, gti, dt=0):
+    """
+    Get the indices of events from different GTIs of the observation.
+
+    This is a generator, yielding the boundaries of each GTI and the
+    corresponding indices in the time array.
+
+    Parameters
+    ----------
+    times : float `np.array`
+        Array of times.
+    gti : [[gti00, gti01], [gti10, gti11], ...]
+        Good time intervals.
+
+    Other parameters
+    ----------------
+    dt : float
+        If times are uniformly binned, this is the binning time.
+
+    Yields
+    -------
+    g0: float
+        Start time of current GTI.
+    g1: float
+        End time of current GTI.
+    startidx: int
+        Start index of the current GTI in the time array.
+    stopidx: int
+        End index of the current GTI in the time array. Note that this is
+        larger by one, so that `time[startidx:stopidx]` returns the correct
+        time interval.
+
+    Examples
+    --------
+    >>> times = [0.1, 0.2, 0.5, 0.8, 1.1]
+    >>> gtis = [[0, 0.55], [0.6, 2.1]]
+    >>> vals = generate_indices_of_gti_boundaries(times, gtis)
+    >>> v0 = next(vals)
+    >>> np.allclose(v0[:2], gtis[0])
+    True
+    >>> np.allclose(v0[2:], [0, 3])
+    True
+    """
+    gti = np.asarray(gti)
+    times = np.asarray(times)
+    startidx, stopidx = gti_border_bins(gti, times, dt=dt)
+
+    for s, e, idx0, idx1 in zip(gti[:, 0], gti[:, 1], startidx, stopidx):
+        yield s, e, idx0, idx1
+
+
+def generate_indices_of_segment_boundaries_unbinned(times, gti, segment_size):
+    """
+    Get the indices of events from different segments of the observation.
+
+    This is a generator, yielding the boundaries of each segment and the
+    corresponding indices in the time array.
+
+    Parameters
+    ----------
+    times : float `np.array`
+        Array of times.
+    gti : [[gti00, gti01], [gti10, gti11], ...]
+        Good time intervals.
+    segment_size : float
+        Length of segments.
+
+    Yields
+    -------
+    t0: float
+        Start time of current segment.
+    t1: float
+        End time of current segment.
+    startidx: int
+        Start index of the current segment in the time array.
+    stopidx: int
+        End index of the current segment in the time array. Note that this is
+        larger by one, so that `time[startidx:stopidx]` returns the correct
+        time interval.
+
+    Examples
+    --------
+    >>> times = [0.1, 0.2, 0.5, 0.8, 1.1]
+    >>> gtis = [[0, 0.55], [0.6, 2.1]]
+    >>> vals = generate_indices_of_segment_boundaries_unbinned(times, gtis, 0.5)
+    >>> v0 = next(vals)
+    >>> np.allclose(v0[:2], [0, 0.5])
+    True
+    >>> # Note: 0.5 is not included in the interval
+    >>> np.allclose(v0[2:], [0, 2])
+    True
+    >>> v1 = next(vals)
+    >>> np.allclose(v1[:2], [0.6, 1.1])
+    True
+    >>> # Again: 1.1 is not included in the interval
+    >>> np.allclose(v1[2:], [3, 4])
+    True
+    """
+    gti = np.asarray(gti)
+    times = np.asarray(times)
+
+    start, stop = time_intervals_from_gtis(gti, segment_size)
+
+    startidx = np.asarray(np.searchsorted(times, start))
+    stopidx = np.asarray(np.searchsorted(times, stop))
+
+    for s, e, idx0, idx1 in zip(start, stop, startidx, stopidx):
+        yield s, e, idx0, idx1
+
+
+def generate_indices_of_segment_boundaries_binned(times, gti, segment_size,
+                                                  dt=None):
+    """
+    Get the indices of binned times from different segments of the observation.
+
+    This is a generator, yielding the boundaries of each segment and the
+    corresponding indices in the time array
+
+    Parameters
+    ----------
+    times : float `np.array`
+        Array of times, uniformly sampled
+    gti : [[gti00, gti01], [gti10, gti11], ...]
+        good time intervals
+    segment_size : float
+        length of segments
+
+    Yields
+    -------
+    t0: float
+        First time value, from the time array, in the current segment
+    t1: float
+        Last time value, from the time array, in the current segment
+    startidx: int
+        Start index of the current segment in the time array
+    stopidx: int
+        End index of the current segment in the time array. Note that this is
+        larger by one, so that `time[startidx:stopidx]` returns the correct
+        time interval.
+
+    Examples
+    --------
+    >>> times = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7]
+    >>> gtis = [[0.05, 0.55]]
+    >>> vals = generate_indices_of_segment_boundaries_binned(times, gtis, 0.5, dt=0.1)
+    >>> v0 = next(vals)
+    >>> np.allclose(v0[:2], [0.05, 0.55])
+    True
+    >>> np.allclose(v0[2:], [0, 5])
+    True
+    """
+    gti = np.asarray(gti)
+    times = np.asarray(times)
+    startidx, stopidx = bin_intervals_from_gtis(gti, segment_size, times,
+                                                dt=dt)
+
+    if dt is None:
+        dt = 0
+    for idx0, idx1 in zip(startidx, stopidx):
+        yield times[idx0] - dt / 2, times[min(idx1, times.size - 1)] - dt / 2,\
+              idx0, idx1
