@@ -10,6 +10,7 @@ import numpy as np
 from astropy.io import fits
 from astropy.table import Table
 from astropy.logger import AstropyUserWarning
+import matplotlib.pyplot as plt
 
 import stingray.utils as utils
 
@@ -584,6 +585,7 @@ def load_events_and_gtis(
     if mission_key not in probe_header:
         mission_key = "TELESCOP"
     mission = probe_header[mission_key].lower()
+
     db = read_mission_info(mission)
     instkey = get_key_from_mission_info(db, "instkey", "INSTRUME")
     instr = mode = None
@@ -608,7 +610,9 @@ def load_events_and_gtis(
     ephem = timeref = timesys = None
 
     if "PLEPHEM" in header:
-        ephem = header["PLEPHEM"].strip().lstrip('JPL-').lower()
+        # For the rare cases where this is a number, e.g. 200, I add `str`
+        # It's supposed to be a string.
+        ephem = str(header["PLEPHEM"]).strip().lstrip('JPL-').lower()
     if "TIMEREF" in header:
         timeref = header["TIMEREF"].strip().lower()
     if "TIMESYS" in header:
@@ -620,8 +624,9 @@ def load_events_and_gtis(
 
     detector_id = None
     ckey = get_key_from_mission_info(db, "ccol", "NONE", instr, mode)
-    if ckey != "NONE":
+    if ckey != "NONE" and ckey in datatable.columns.names:
         detector_id = datatable.field(ckey)
+
     det_number = None if detector_id is None else list(set(detector_id))
 
     timezero = np.longdouble(0.)
@@ -665,6 +670,9 @@ def load_events_and_gtis(
         additional_columns = [pi_col]
     if pi_col not in additional_columns:
         additional_columns.append(pi_col)
+    # If data were already calibrated, use this!
+    if "energy" not in additional_columns:
+        additional_columns.append("energy")
 
     additional_data = _get_additional_data(datatable, additional_columns)
     hdulist.close()
@@ -688,7 +696,7 @@ def load_events_and_gtis(
     returns.gti_list = gti_list
     returns.pi_list = pi
     returns.cal_pi_list = cal_pi
-    if "energy" in additional_data:
+    if "energy" in additional_data and np.any(additional_data["energy"] > 0.):
         returns.energy_list = additional_data["energy"]
     else:
         try:
@@ -728,14 +736,7 @@ def mkdir_p(path):  # pragma: no cover
     .. [so-mkdir] http://stackoverflow.com/questions/600268/mkdir-p-functionality-in-python
     """
     import os
-    import errno
-    try:
-        os.makedirs(path)
-    except OSError as exc:  # Python >2.5
-        if exc.errno == errno.EEXIST and os.path.isdir(path):
-            pass
-        else:
-            raise
+    os.makedirs(path, exist_ok=True)
 
 
 def read_header_key(fits_file, key, hdu=1):
@@ -889,545 +890,6 @@ def split_numbers(number, shift=0):
     return np.double(number_I) / 10**shift, np.double(number_F) / 10**shift
 
 
-def _save_pickle_object(object, filename):
-    """
-    Save a class object in pickle format.
-
-    Parameters
-    ----------
-    object: class instance
-        A class object whose attributes are saved in a
-        dictionary format
-
-    filename: str
-        Name of the file in which object is saved
-    """
-
-    with open(filename, "wb") as f:
-        pickle.dump(object, f)
-
-
-def _retrieve_pickle_object(filename):
-    """
-    Retrieves a pickled class object.
-
-    Parameters
-    ----------
-    filename: str
-        Name of the file in which object is saved
-
-    Returns
-    -------
-    data: class object
-    """
-
-    with open(filename, "rb") as f:
-        return pickle.load(f)
-
-
-def _save_hdf5_object(object, filename):
-    """
-    Save a class object in hdf5 format.
-
-    Parameters
-    ----------
-    object: class instance
-        A class object whose attributes are saved in a
-        dictionary format
-
-    filename: str
-        Name of the file in which object is saved
-    """
-
-    items = vars(object)
-    attrs = [name for name in items if items[name] is not None]
-
-    with h5py.File(filename, 'w') as hf:
-        for attr in attrs:
-            data = items[attr]
-
-            # If data is a single number, store as an attribute.
-            if _isattribute(data):
-                if isinstance(data, np.longdouble):
-                    data_I, data_F = split_numbers(data)
-                    names = [attr + '_I', attr + '_F']
-                    hf.attrs[names[0]] = data_I
-                    hf.attrs[names[1]] = data_F
-                else:
-                    hf.attrs[attr] = data
-
-            # If data is an array or list, create a dataset.
-            else:
-                try:
-                    if isinstance(data[0], np.longdouble):
-                        data_I, data_F = split_numbers(data)
-                        names = [attr + '_I', attr + '_F']
-                        hf.create_dataset(names[0], data=data_I)
-                        hf.create_dataset(names[1], data=data_F)
-                    else:
-                        hf.create_dataset(attr, data=data)
-                except IndexError:
-                    # To account for numpy arrays of type 'None' (0-d)
-                    pass
-
-
-def _retrieve_hdf5_object(filename):
-    """
-    Retrieves an hdf5 format class object.
-
-    Parameters
-    ----------
-    filename: str
-        The name of file with which object was saved
-
-    Returns
-    -------
-    data: dictionary
-        Loads the data from an hdf5 object file and returns
-        in dictionary format.
-    """
-
-    with h5py.File(filename, 'r') as hf:
-        dset_keys = hf.keys()
-        attr_keys = hf.attrs.keys()
-        data = {}
-
-        dset_copy = list(dset_keys)[:]
-        for key in dset_keys:
-
-            # Make sure key hasn't been removed
-            if key in dset_copy:
-                # Longdouble case
-                if key[-2:] in ['_I', '_F']:
-                    m_key = key[:-2]
-                    # Add integer and float parts
-                    data[m_key] = np.longdouble(hf[m_key + '_I'][()])
-                    data[m_key] += np.longdouble(hf[m_key + '_F'][()])
-                    # Remove integer and float parts from attributes
-                    dset_copy.remove(m_key + '_I')
-                    dset_copy.remove(m_key + '_F')
-                else:
-                    data[key] = hf[key][()]
-
-        attr_copy = list(attr_keys)[:]
-        for key in attr_keys:
-
-            # Make sure key hasn't been removed
-            if key in attr_copy:
-                # Longdouble case
-                if key[-2:] in ['_I', '_F']:
-                    m_key = key[:-2]
-                    # Add integer and float parts
-                    data[m_key] = np.longdouble(hf.attrs[m_key + '_I'])
-                    data[m_key] += np.longdouble(hf.attrs[m_key + '_F'])
-                    # Remove integer and float parts from attributes
-                    attr_copy.remove(m_key + '_I')
-                    attr_copy.remove(m_key + '_F')
-                else:
-                    data[key] = hf.attrs[key]
-
-    return data
-
-
-def _save_ascii_object(object, filename, fmt="%.18e", **kwargs):
-    """
-    Save an array to a text file.
-
-    Parameters
-    ----------
-    object : numpy.ndarray
-        An array with the data to be saved
-
-    filename : str
-        The file name to save to
-
-    fmt : str or sequence of strs, optional
-        Use for formatting of columns. See `numpy.savetxt` documentation
-        for details.
-
-    Other Parameters
-    ----------------
-    kwargs : any keyword argument taken by `numpy.savetxt`
-
-    """
-
-    try:
-        np.savetxt(filename, object, fmt=fmt, **kwargs)
-    except TypeError:
-        raise Exception("Formatting of columns not recognized! Use 'fmt' "
-                        "to format columns including strings or mixed types!")
-
-    pass
-
-
-def _retrieve_ascii_object(filename, **kwargs):
-    """
-    Helper function to retrieve ascii objects from file.
-    Uses astropy.Table for reading and storing the data.
-
-    Parameters
-    ----------
-    filename : str
-        The name of the file with the data to be retrieved.
-
-    Other Parameters
-    -----------------------------
-    usecols : {int | iterable}
-        The indices of the columns in the file to be returned.
-        By default, all columns will be returned
-
-    skiprows : int
-        The number of rows at the beginning to skip
-        By default, no rows will be skipped.
-
-    names : iterable
-        A list of column names to be attached to the columns.
-        By default, no column names are added, unless they are specified
-        in the file header and can be read by astropy.Table.read
-        automatically.
-
-    Returns
-    -------
-    data : astropy.Table object
-        An astropy.Table object with the data from the file
-    """
-    if not isinstance(filename, str):
-        raise TypeError("filename must be string!")
-
-    if 'usecols' in list(kwargs.keys()):
-        if np.size(kwargs['usecols']) != 2:
-            raise ValueError("Need to define two columns")
-        usecols = kwargs["usecols"]
-    else:
-        usecols = None
-
-    if 'skiprows' in list(kwargs.keys()):
-        assert isinstance(kwargs["skiprows"], int)
-        skiprows = kwargs["skiprows"]
-    else:
-        skiprows = 0
-
-    if "names" in list(kwargs.keys()):
-        names = kwargs["names"]
-    else:
-        names = None
-
-    data = Table.read(filename, data_start=skiprows,
-                      names=names, format="ascii")
-
-    if usecols is None:
-        return data
-    else:
-        colnames = np.array(data.colnames)
-        cols = colnames[usecols]
-
-        return data[cols]
-
-
-def _save_fits_object(object, filename, **kwargs):
-    """
-    Save a class object in fits format.
-
-    Parameters
-    ----------
-    object: class instance
-        A class object whose attributes would be saved in a dictionary format.
-
-    filename: str
-        The file name to save to
-
-    Additional Keyword Parameters
-    -----------------------------
-    tnames: str iterable
-        The names of HDU tables. For instance, in case of eventlist,
-        tnames could be ['EVENTS', 'GTI']
-
-    colsassign: dictionary iterable
-        This indicates the correct tables to which to assign columns
-        to. If this is None or if a column is not provided, it/they will
-        be assigned to the first table.
-
-        For example, [{'gti':'GTI'}] indicates that gti values should be
-        stored in GTI table.
-    """
-
-    tables = []
-
-    if 'colsassign' in list(kwargs.keys()):
-        colsassign = kwargs['colsassign']
-        iscolsassigned = True
-    else:
-        iscolsassigned = False
-
-    if 'tnames' in list(kwargs.keys()):
-        tables = kwargs['tnames']
-    else:
-        tables = ['MAIN']
-
-    items = vars(object)
-    attrs = [name for name in items if items[name] is not None]
-
-    cols = []
-    hdrs = []
-
-    for t in tables:
-        cols.append([])
-        hdrs.append(fits.Header())
-
-    for attr in attrs:
-        data = items[attr]
-
-        # Get the index of table to which column belongs
-        if iscolsassigned and attr in colsassign.keys():
-            index = tables.index(colsassign[attr])
-        else:
-            index = 0
-
-        # If data is a single number, store as metadata
-        if _isattribute(data):
-            if isinstance(data, np.longdouble):
-                # Longdouble case. Split and save integer and float parts
-                data_I, data_F = split_numbers(data)
-                names = [attr + '_I', attr + '_F']
-                hdrs[index][names[0]] = data_I
-                hdrs[index][names[1]] = data_F
-            else:
-                # Normal case. Save as it is
-                hdrs[index][attr] = data
-
-        # If data is an array or list, insert as table column
-        else:
-            try:
-                if isinstance(data[0], np.longdouble):
-                    # Longdouble case. Split and save integer and float parts
-                    data_I, data_F = split_numbers(data)
-                    names = [attr + '_I', attr + '_F']
-                    cols[index].append(
-                        fits.Column(name=names[0],
-                                    format='D',
-                                    array=data_I))
-                    cols[index].append(
-                        fits.Column(name=names[1],
-                                    format='D',
-                                    array=data_F))
-                else:
-                    # Normal case. Save as it is
-                    cols[index].append(
-                        fits.Column(name=attr,
-                                    format=_lookup_format(data[0]),
-                                    array=data))
-            except IndexError:
-                # To account for numpy arrays of type 'None' (0-d)
-                pass
-
-    tbhdu = fits.HDUList()
-
-    # Create binary tables
-    for i in range(0, len(tables)):
-        if len(cols[i]) > 0:
-            tbhdu.append(fits.BinTableHDU.from_columns(cols[i],
-                                                       header=hdrs[i],
-                                                       name=tables[i]))
-
-    tbhdu.writeto(filename)
-
-
-def _retrieve_fits_object(filename, **kwargs):
-    """
-    Retrieves a fits format class object.
-
-    Parameters
-    ----------
-    filename: str
-        The name of file with which object was saved
-
-    Other Parameters
-    ----------------
-    cols: str iterable
-        The names of columns to extract from fits tables.
-
-    Returns
-    -------
-    data: dictionary
-        Loads the data from a fits object file and returns
-        in dictionary format.
-    """
-
-    data = {}
-
-    if 'cols' in list(kwargs.keys()):
-        cols = [col.upper() for col in kwargs['cols']]
-    else:
-        cols = []
-
-    with fits.open(filename, memmap=False, ignore_missing_end=True) as hdulist:
-        fits_cols = []
-
-        # Get columns from all tables
-        for i in range(1, len(hdulist)):
-            fits_cols.append([h.upper() for h in hdulist[i].data.names])
-
-        for c in cols:
-            for i in range(0, len(fits_cols)):
-                # .upper() is used because `fits` stores values in upper case
-                hdr_keys = [h.upper() for h in hdulist[i + 1].header.keys()]
-
-                # Longdouble case. Check for columns
-                if c + '_I' in fits_cols[i] or c + '_F' in fits_cols[i]:
-                    if c not in data.keys():
-                        data[c] = np.longdouble(hdulist[i + 1].data[c + '_I'])
-                        data[c] += np.longdouble(hdulist[i + 1].data[c + '_F'])
-
-                # Longdouble case. Check for header keys
-                if c + '_I' in hdr_keys or c + '_F' in hdr_keys:
-                    if c not in data.keys():
-                        data[c] = \
-                            np.longdouble(hdulist[i + 1].header[c + '_I'])
-                        data[c] += \
-                            np.longdouble(hdulist[i + 1].header[c + '_F'])
-
-                # Normal case. Check for columns
-                elif c in fits_cols[i]:
-                    data[c] = hdulist[i + 1].data[c]
-
-                # Normal case. Check for header keys
-                elif c in hdr_keys:
-                    data[c] = hdulist[i + 1].header[c]
-        hdulist.close()
-    return data
-
-
-def _lookup_format(var):
-    """
-    Looks up relevant format in fits.
-
-    Parameters
-    ----------
-    var : object
-        An object to look up in the table
-
-    Returns
-    -------
-    lookup : str
-        The str describing the type of ``var``
-    """
-
-    lookup = {"<type 'int'>": "J", "<type 'float'>": "E",
-              "<type 'numpy.int64'>": "K", "<type 'numpy.float64'>": "D",
-              "<type 'numpy.float128'>": "D", "<type 'str'>": "30A",
-              "<type 'bool'": "L"}
-
-    form = type(var)
-
-    try:
-        return lookup[str(form)]
-    except KeyError:
-        # If an entry is not contained in lookup dictionary
-        return "D"
-
-
-def _isattribute(data):
-    """
-    Check if data is a single number or an array.
-
-    Parameters
-    ----------
-    data : object
-        The object to be checked.
-
-    Returns:
-        bool
-        True if the data is a single number, False if it is an iterable.
-    """
-
-    if isinstance(data, Iterable) and not isinstance(data, (str, bytes)):
-        return False
-    else:
-        return True
-
-
-def write(input_, filename, format_='pickle', **kwargs):
-    """
-    Pickle a class instance. For parameters depending on
-    ``format_``, see individual function definitions.
-
-    Parameters
-    ----------
-    object: a class instance
-        The object to be stored
-
-    filename: str
-        The name of the file to be created
-
-    format_: str
-        The format in which to store file. Formats supported
-        are ``pickle``, ``hdf5``, ``ascii`` or ``fits``
-    """
-
-    if format_ == 'pickle':
-        _save_pickle_object(input_, filename)
-
-    elif format_ == 'hdf5':
-        if _H5PY_INSTALLED:
-            _save_hdf5_object(input_, filename)
-        else:
-            utils.simon('h5py not installed, using pickle instead'
-                        'to save object.')
-            _save_pickle_object(input_, filename.split('.')[0] +
-                                '.pickle')
-
-    elif format_ == 'ascii':
-        _save_ascii_object(input_, filename, **kwargs)
-
-    elif format_ == 'fits':
-        _save_fits_object(input_, filename, **kwargs)
-
-    else:
-        utils.simon('Format not understood.')
-
-
-def read(filename, format_='pickle', **kwargs):
-    """
-    Return a saved class instance.
-
-    Parameters
-    ----------
-    filename: str
-        The name of the file to be retrieved.
-
-    format_: str
-        The format used to store file. Supported formats are
-        pickle, hdf5, ascii or fits.
-
-    Returns
-    -------
-    data : {``object`` | ``astropy.table`` | ``dict``}
-
-        * If ``format_`` is ``pickle``, an object is returned.
-        * If ``format_`` is ``ascii``, `astropy.table` object is returned.
-        * If ``format_`` is ``hdf5`` or 'fits``, a dictionary object is returned.
-    """
-
-    if format_ == 'pickle':
-        return _retrieve_pickle_object(filename)
-
-    elif format_ == 'hdf5':
-        if _H5PY_INSTALLED:
-            return _retrieve_hdf5_object(filename)
-        else:
-            utils.simon('h5py not installed, cannot read an'
-                        'hdf5 object.')
-
-    elif format_ == 'ascii':
-        return _retrieve_ascii_object(filename, **kwargs)
-
-    elif format_ == 'fits':
-        return _retrieve_fits_object(filename, **kwargs)
-
-    else:
-        utils.simon('Format not understood.')
-
-
 def savefig(filename, **kwargs):
     """
     Save a figure plotted by ``matplotlib``.
@@ -1449,11 +911,6 @@ def savefig(filename, **kwargs):
         remove the undesirable whitepace around the image.
     """
 
-    try:
-        import matplotlib.pyplot as plt
-    except ImportError:
-        raise ImportError("Matplotlib required for savefig()")
-
     if not plt.fignum_exists(1):
         utils.simon("use ``plot`` function to plot the image first and "
                     "then use ``savefig`` to save the figure.")
