@@ -4,31 +4,43 @@ Definition of :class::class:`Lightcurve`.
 :class::class:`Lightcurve` is used to create light curves out of photon counting data
 or to save existing light curves in a class that's easy to use.
 """
-import warnings
+import os
 import logging
+import warnings
+import pickle
+
 import numpy as np
-import stingray.io as io
+import matplotlib.pyplot as plt
+from astropy.table import Table
+from astropy.time import TimeDelta, Time
+from astropy import units as u
+
+from stingray.base import StingrayTimeseries
 import stingray.utils as utils
 from stingray.exceptions import StingrayError
-from stingray.utils import simon, assign_value_if_none, baseline_als
-from stingray.utils import poisson_symmetrical_errors
-from stingray.gti import cross_two_gtis, join_gtis, gti_border_bins
-from stingray.gti import check_gtis, create_gti_mask_complete, create_gti_mask, bin_intervals_from_gtis
+from stingray.gti import (bin_intervals_from_gtis, check_gtis, create_gti_mask,
+                          cross_two_gtis, gti_border_bins, join_gtis)
+from stingray.utils import (assign_value_if_none, baseline_als,
+                            poisson_symmetrical_errors, simon, interpret_times)
+from stingray.io import lcurve_from_fits
 
 __all__ = ["Lightcurve"]
 
 valid_statistics = ["poisson", "gauss", None]
 
 
-class Lightcurve(object):
+class Lightcurve(StingrayTimeseries):
     """
     Make a light curve object from an array of time stamps and an
     array of counts.
 
     Parameters
     ----------
-    time: iterable
-        A list or array of time stamps for a light curve
+    time: Iterable, `:class:astropy.time.Time`, or `:class:astropy.units.Quantity` object
+        A list or array of time stamps for a light curve. Must be a type that
+        can be cast to `:class:np.array` or `:class:List` of floats, or that
+        has a `value` attribute that does (e.g. a
+        `:class:astropy.units.Quantity` or `:class:astropy.time.Time` object).
 
     counts: iterable, optional, default ``None``
         A list or array of the counts in each bin corresponding to the
@@ -73,6 +85,18 @@ class Lightcurve(object):
         countrate_err if input_counts is True) will _not_ be stored in memory,
         but calculated every time they are requested.
 
+    mission : str
+        Mission that recorded the data (e.g. NICER)
+
+    instr : str
+        Instrument onboard the mission
+
+    header : str
+        The full header of the original FITS file, if relevant
+
+    **other_kw :
+        Used internally. Any other keyword arguments will be ignored
+
     Attributes
     ----------
     time: numpy.ndarray
@@ -128,13 +152,36 @@ class Lightcurve(object):
         uncertainties and other statistical values appropriately.
         It propagates to Spectrum classes.
 
+    mission : str
+        Mission that recorded the data (e.g. NICER)
+
+    instr : str
+        Instrument onboard the mission
+
+    detector_id : iterable
+        The detector that recoded each photon, if relevant (e.g. XMM, Chandra)
+
+    header : str
+        The full header of the original FITS file, if relevant
+
     """
+    main_array_attr = "time"
+
     def __init__(self, time, counts, err=None, input_counts=True,
                  gti=None, err_dist='poisson', mjdref=0, dt=None,
-                 skip_checks=False, low_memory=False):
+                 skip_checks=False, low_memory=False, mission=None,
+                 instr=None, header=None, **other_kw):
+
+        StingrayTimeseries.__init__(self)
+
+        if other_kw != {}:
+            warnings.warn(f"Unrecognized keywords: {list(other_kw.keys())}")
+
+        time, mjdref = interpret_times(time, mjdref=mjdref)
 
         time = np.asarray(time)
         counts = np.asarray(counts)
+
         if err is not None:
             err = np.asarray(err)
 
@@ -170,9 +217,9 @@ class Lightcurve(object):
         self._time = time
 
         if dt is None:
-            logging.warning("Computing the bin time ``dt``. This can take "
-                            "time. If you know the bin time, please specify it"
-                            " at light curve creation")
+            logging.info("Computing the bin time ``dt``. This can take "
+                         "time. If you know the bin time, please specify it"
+                         " at light curve creation")
             dt = np.median(np.diff(self._time))
 
         self.dt = dt
@@ -195,6 +242,9 @@ class Lightcurve(object):
         self._bin_lo = None
         self._bin_hi = None
         self._n = None
+        self.mission = mission
+        self.instr = instr
+        self.header = header
 
         self.input_counts = input_counts
         self.low_memory = low_memory
@@ -207,6 +257,11 @@ class Lightcurve(object):
 
         if not skip_checks:
             self.check_lightcurve()
+        if os.name == 'nt':
+            warnings.warn(
+                "On Windows, the size of an integer is 32 bits. "
+                "To avoid integer overflow, I'm converting the input array to float")
+            counts = counts.astype(float)
 
     @property
     def time(self):
@@ -371,10 +426,10 @@ class Lightcurve(object):
         return self._bin_hi
 
     def initial_optional_checks(self, time, counts, err):
-        logging.warning("Checking if light curve is well behaved. This "
-                        "can take time, so if you are sure it is already "
-                        "sorted, specify skip_checks=True at light curve "
-                        "creation.")
+        logging.info("Checking if light curve is well behaved. This "
+                     "can take time, so if you are sure it is already "
+                     "sorted, specify skip_checks=True at light curve "
+                     "creation.")
 
         if not np.all(np.isfinite(time)):
             raise ValueError("There are inf or NaN values in "
@@ -384,7 +439,7 @@ class Lightcurve(object):
             raise ValueError("There are inf or NaN values in "
                              "your counts array!")
 
-        logging.warning("Checking if light curve is sorted.")
+        logging.info("Checking if light curve is sorted.")
         dt_array = np.diff(time)
         unsorted = np.any(dt_array < 0)
 
@@ -408,63 +463,21 @@ class Lightcurve(object):
 
         check_gtis(self.gti)
 
-        dt_array = []
-        for g in self.gti:
-            mask = create_gti_mask(self.time, [g], dt=self.dt)
-            t = self.time[mask]
-            dt_array.extend(np.diff(t))
-        dt_array = np.asarray(dt_array)
+        idxs = np.searchsorted(self.time, self.gti)
+        uneven = False
+        for idx in range(idxs.shape[0]):
+            istart, istop = idxs[idx, 0], min(idxs[idx, 1], self.time.size - 1)
 
-        if not (np.allclose(dt_array, np.repeat(self.dt, dt_array.shape[0]))):
+            local_diff = np.diff(self.time[istart:istop])
+            if np.any(~np.isclose(local_diff, self.dt)):
+                uneven = True
+
+                break
+        if uneven:
             simon("Bin sizes in input time array aren't equal throughout! "
                   "This could cause problems with Fourier transforms. "
                   "Please make the input time evenly sampled.")
 
-    def change_mjdref(self, new_mjdref):
-        """Change the MJD reference time (MJDREF) of the light curve.
-
-        Times will be now referred to this new MJDREF
-
-        Parameters
-        ----------
-        new_mjdref : float
-            New MJDREF
-
-        Returns
-        -------
-        new_lc : lightcurve.Lightcurve object
-            The new LC shifted by MJDREF
-        """
-        time_shift = -(new_mjdref - self.mjdref) * 86400
-
-        new_lc = self.shift(time_shift)
-        new_lc.mjdref = new_mjdref
-        return new_lc
-
-    def shift(self, time_shift):
-        """
-        Shift the light curve and the GTIs in time.
-
-        Parameters
-        ----------
-        time_shift: float
-            The time interval by which the light curve will be shifted (in
-            the same units as the time array in :class:`Lightcurve`
-
-        Returns
-        -------
-        new_lc : lightcurve.Lightcurve object
-            The new LC shifted by ``time_shift``
-
-        """
-        new_lc = Lightcurve(self.time + time_shift, self.counts,
-                            err=self.counts_err,
-                            gti=self.gti + time_shift, mjdref=self.mjdref,
-                            dt=self.dt, err_dist=self.err_dist,
-                            skip_checks=True)
-
-        return new_lc
-
     def _operation_with_other_lc(self, other, operation):
         """
         Helper method to codify an operation of one light curve with another (e.g. add, subtract, ...).
@@ -510,10 +523,10 @@ class Lightcurve(object):
                   "We are setting the errors to zero to avoid complications.")
             new_counts_err = np.zeros_like(new_counts)
         elif self.err_dist.lower() in valid_statistics:
-                new_counts_err = \
-                    np.sqrt(np.add(self.counts_err[mask_self]**2,
-                                   other.counts_err[mask_other]**2))
-            # More conditions can be implemented for other statistics
+            new_counts_err = \
+                np.sqrt(np.add(self.counts_err[mask_self]**2,
+                               other.counts_err[mask_other]**2))
+        # More conditions can be implemented for other statistics
         else:
             raise StingrayError("Statistics not recognized."
                                 " Please use one of these: "
@@ -546,8 +559,8 @@ class Lightcurve(object):
         >>> lc1 = Lightcurve(time, count1, gti=gti1, dt=5)
         >>> lc2 = Lightcurve(time, count2, gti=gti2, dt=5)
         >>> lc = lc1 + lc2
-        >>> lc.counts
-        array([ 900, 1300, 1200])
+        >>> np.allclose(lc.counts, [ 900, 1300, 1200])
+        True
         """
 
         return self._operation_with_other_lc(other, np.add)
@@ -571,12 +584,12 @@ class Lightcurve(object):
         >>> count1 = [600, 1200, 800]
         >>> count2 = [300, 100, 400]
         >>> gti1 = [[0, 35]]
-        >>> gti2 = [[0, 40]]
-        >>> lc1 = Lightcurve(time, count1, gti=gti1, dt=5)
-        >>> lc2 = Lightcurve(time, count2, gti=gti2, dt=5)
+        >>> gti2 = [[5, 40]]
+        >>> lc1 = Lightcurve(time, count1, gti=gti1, dt=10)
+        >>> lc2 = Lightcurve(time, count2, gti=gti2, dt=10)
         >>> lc = lc1 - lc2
-        >>> lc.counts
-        array([ 300, 1100,  400])
+        >>> np.allclose(lc.counts, [ 300, 1100,  400])
+        True
         """
 
         return self._operation_with_other_lc(other, np.subtract)
@@ -596,8 +609,8 @@ class Lightcurve(object):
         >>> lc1 = Lightcurve(time, count1)
         >>> lc2 = Lightcurve(time, count2)
         >>> lc_new = -lc1 + lc2
-        >>> lc_new.counts
-        array([100, 100, 100])
+        >>> np.allclose(lc_new.counts, [100, 100, 100])
+        True
         """
         lc_new = Lightcurve(self.time, -1 * self.counts,
                             err=self.counts_err, gti=self.gti,
@@ -647,10 +660,10 @@ class Lightcurve(object):
         >>> time = [1, 2, 3, 4, 5, 6, 7, 8, 9]
         >>> count = [11, 22, 33, 44, 55, 66, 77, 88, 99]
         >>> lc = Lightcurve(time, count, dt=1)
-        >>> lc[2]
-        33
-        >>> lc[:2].counts
-        array([11, 22])
+        >>> np.isclose(lc[2], 33)
+        True
+        >>> np.allclose(lc[:2].counts, [11, 22])
+        True
         """
         if isinstance(index, (int, np.integer)):
             return self.counts[index]
@@ -671,8 +684,12 @@ class Lightcurve(object):
                 new_gti = cross_two_gtis(new_gti, new_gt1)
             new_gti = cross_two_gtis(self.gti, new_gti)
 
-            return Lightcurve(new_time, new_counts, mjdref=self.mjdref,
-                              gti=new_gti, dt=self.dt, skip_checks=True)
+            lc = Lightcurve(new_time, new_counts, mjdref=self.mjdref,
+                            gti=new_gti, dt=self.dt, skip_checks=True,
+                            err_dist=self.err_dist)
+            if self._counts_err is not None:
+                lc._counts_err = self._counts_err[start:stop:step]
+            return lc
         else:
             raise IndexError("The index must be either an integer or a slice "
                              "object !")
@@ -739,7 +756,6 @@ class Lightcurve(object):
     @staticmethod
     def make_lightcurve(toa, dt, tseg=None, tstart=None, gti=None, mjdref=0,
                         use_hist=False):
-
         """
         Make a light curve out of photon arrival times, with a given time resolution ``dt``.
         Note that ``dt`` should be larger than the native time resolution of the instrument
@@ -781,6 +797,7 @@ class Lightcurve(object):
         lc: :class:`Lightcurve` object
             A :class:`Lightcurve` object with the binned light curve
         """
+        toa, mjdref = interpret_times(toa, mjdref=mjdref)
 
         toa = np.sort(np.asarray(toa))
         # tstart is an optional parameter to set a starting time for
@@ -799,7 +816,11 @@ class Lightcurve(object):
 
         logging.info("make_lightcurve: tseg: " + str(tseg))
 
-        timebin = np.int64(tseg / dt)
+        timebin = int(tseg / dt)
+        # If we are missing the next bin by just 1%, let's round up:
+        if tseg / dt - timebin >= 0.99:
+            timebin += 1
+
         logging.info("make_lightcurve: timebin:  " + str(timebin))
 
         tend = tstart + timebin * dt
@@ -887,7 +908,7 @@ class Lightcurve(object):
                             skip_checks=True)
         return lc_new
 
-    def join(self, other):
+    def join(self, other, skip_checks=False):
         """
         Join two lightcurves into a single object.
 
@@ -904,6 +925,9 @@ class Lightcurve(object):
         ----------
         other : :class:`Lightcurve` object
             The other :class:`Lightcurve` object which is supposed to be joined with.
+        skip_checks: bool
+            If True, the user specifies that data are already sorted and
+            contain no infinite or nan points. Use at your own risk.
 
         Returns
         -------
@@ -921,8 +945,8 @@ class Lightcurve(object):
         >>> lc = lc1.join(lc2)
         >>> lc.time
         array([ 5, 10, 15, 20, 25, 30])
-        >>> lc.counts
-        array([ 300,  100,  400,  600, 1200,  800])
+        >>> np.allclose(lc.counts, [ 300,  100,  400,  600, 1200,  800])
+        True
         """
         if self.mjdref != other.mjdref:
             warnings.warn("MJDref is different in the two light curves")
@@ -1001,7 +1025,7 @@ class Lightcurve(object):
         gti = join_gtis(self.gti, other.gti)
 
         lc_new = Lightcurve(new_time, new_counts, err=new_counts_err, gti=gti,
-                            mjdref=self.mjdref, dt=self.dt)
+                            mjdref=self.mjdref, dt=self.dt, skip_checks=skip_checks)
 
         return lc_new
 
@@ -1043,17 +1067,16 @@ class Lightcurve(object):
         >>> count = [10, 20, 30, 40, 50, 60, 70, 80, 90]
         >>> lc = Lightcurve(time, count, dt=1)
         >>> lc_new = lc.truncate(start=2, stop=8)
-        >>> lc_new.counts
-        array([30, 40, 50, 60, 70, 80])
+        >>> np.allclose(lc_new.counts, [30, 40, 50, 60, 70, 80])
+        True
         >>> lc_new.time
         array([3, 4, 5, 6, 7, 8])
         >>> # Truncation can also be done by time values
         >>> lc_new = lc.truncate(start=6, method='time')
         >>> lc_new.time
         array([6, 7, 8, 9])
-        >>> lc_new.counts
-        array([60, 70, 80, 90])
-
+        >>> np.allclose(lc_new.counts, [60, 70, 80, 90])
+        True
         """
 
         if not isinstance(method, str):
@@ -1079,7 +1102,7 @@ class Lightcurve(object):
                                         time_new[-1] + 0.5 * self.dt]]))
 
         return Lightcurve(time_new, counts_new, err=counts_err_new, gti=gti,
-                          dt=self.dt)
+                          dt=self.dt, skip_checks=True)
 
     def _truncate_by_time(self, start, stop):
         """Helper method for truncation using time values.
@@ -1139,10 +1162,11 @@ class Lightcurve(object):
         --------
         >>> time = np.array([1, 2, 3, 6, 7, 8, 11, 12, 13])
         >>> counts = np.random.rand(time.shape[0])
-        >>> lc = Lightcurve(time, counts, dt=1)
+        >>> lc = Lightcurve(time, counts, dt=1, skip_checks=True)
         >>> split_lc = lc.split(1.5)
 
         """
+
         # calculate the difference between time bins
         tdiff = np.diff(self.time)
         # find all distances between time bins that are larger than `min_gap`
@@ -1150,18 +1174,17 @@ class Lightcurve(object):
 
         # tolerance for the newly created GTIs: Note that this seems to work
         # with a tolerance of 2, but not if I substitute 10. I don't know why
-        epsilon = np.min(tdiff)/2.0
+        epsilon = np.min(tdiff) / 2.0
 
         # calculate new GTIs
-        gti_start = np.hstack([self.time[0]-epsilon, self.time[gap_idx+1]-epsilon])
-        gti_stop = np.hstack([self.time[gap_idx]+epsilon, self.time[-1]+epsilon])
+        gti_start = np.hstack([self.time[0] - epsilon, self.time[gap_idx + 1] - epsilon])
+        gti_stop = np.hstack([self.time[gap_idx] + epsilon, self.time[-1] + epsilon])
 
         gti = np.vstack([gti_start, gti_stop]).T
         if hasattr(self, 'gti') and self.gti is not None:
             gti = cross_two_gtis(self.gti, gti)
-        self.gti = gti
 
-        lc_split = self.split_by_gti(min_points=min_points)
+        lc_split = self.split_by_gti(gti, min_points=min_points)
         return lc_split
 
     def sort(self, reverse=False):
@@ -1185,8 +1208,8 @@ class Lightcurve(object):
         >>> lc_new = lc.sort()
         >>> lc_new.time
         array([1, 2, 3])
-        >>> lc_new.counts
-        array([100, 200, 300])
+        >>> np.allclose(lc_new.counts, [100, 200, 300])
+        True
 
         Returns
         -------
@@ -1234,8 +1257,8 @@ class Lightcurve(object):
         >>> lc_new = lc.sort_counts()
         >>> lc_new.time
         array([2, 1, 3])
-        >>> lc_new.counts
-        array([100, 200, 300])
+        >>> np.allclose(lc_new.counts, [100, 200, 300])
+        True
         """
 
         new_counts, new_time, new_counts_err = \
@@ -1248,7 +1271,13 @@ class Lightcurve(object):
 
         return new_lc
 
-    def estimate_chunk_length(self, min_total_counts=100, min_time_bins=100):
+    def estimate_chunk_length(self, *args, **kwargs):
+        """Deprecated alias of estimate_segment_size.
+        """
+        warnings.warn("This function was renamed to estimate_segment_size", DeprecationWarning)
+        return self.estimate_segment_size(*args, **kwargs)
+
+    def estimate_segment_size(self, min_total_counts=100, min_time_bins=100):
         """Estimate a reasonable segment length for chunk-by-chunk analysis.
 
         Choose a reasonable length for time segments, given a minimum number of total
@@ -1266,7 +1295,7 @@ class Lightcurve(object):
 
         Returns
         -------
-        chunk_length : float
+        segment_size : float
             The length of the light curve chunks that satisfies the conditions
 
         Examples
@@ -1275,38 +1304,48 @@ class Lightcurve(object):
         >>> time = np.arange(150)
         >>> count = np.zeros_like(time) + 3
         >>> lc = Lightcurve(time, count, dt=1)
-        >>> lc.estimate_chunk_length(min_total_counts=10, min_time_bins=3)
+        >>> lc.estimate_segment_size(min_total_counts=10, min_time_bins=3)
         4.0
-        >>> lc.estimate_chunk_length(min_total_counts=10, min_time_bins=5)
+        >>> lc.estimate_segment_size(min_total_counts=10, min_time_bins=5)
         5.0
         >>> count[2:4] = 1
         >>> lc = Lightcurve(time, count, dt=1)
-        >>> lc.estimate_chunk_length(min_total_counts=3, min_time_bins=1)
-        4.0
+        >>> lc.estimate_segment_size(min_total_counts=3, min_time_bins=1)
+        3.0
+        >>> # A slightly more complex example
+        >>> dt=0.2
+        >>> time = np.arange(0, 1000, dt)
+        >>> counts = np.random.poisson(100, size=len(time))
+        >>> lc = Lightcurve(time, counts, dt=dt)
+        >>> lc.estimate_segment_size(100, 2)
+        0.4
+        >>> min_total_bins = 40
+        >>> lc.estimate_segment_size(100, 40)
+        8.0
         """
 
         rough_estimate = np.ceil(min_total_counts / self.meancounts) * self.dt
 
-        chunk_length = np.max([rough_estimate, min_time_bins * self.dt])
+        segment_size = np.max([rough_estimate, min_time_bins * self.dt])
 
         keep_searching = True
         while keep_searching:
             start_times, stop_times, results = \
-                self.analyze_lc_chunks(chunk_length, np.sum)
+                self.analyze_lc_chunks(segment_size, np.sum)
             mincounts = np.min(results)
             if mincounts >= min_total_counts:
                 keep_searching = False
             else:
-                chunk_length *= np.ceil(min_total_counts / mincounts) * self.dt
+                segment_size += self.dt
 
-        return chunk_length
+        return segment_size
 
-    def analyze_lc_chunks(self, chunk_length, func, fraction_step=1, **kwargs):
+    def analyze_lc_chunks(self, segment_size, func, fraction_step=1, **kwargs):
         """Analyze segments of the light curve with any function.
 
         Parameters
         ----------
-        chunk_length : float
+        segment_size : float
             Length in seconds of the light curve segments
         func : function
             Function accepting a :class:`Lightcurve` object as single argument, plus
@@ -1317,9 +1356,9 @@ class Lightcurve(object):
         Other parameters
         ----------------
         fraction_step : float
-            If the step is not a full ``chunk_length`` but less (e.g. a moving window),
-            this indicates the ratio between step step and ``chunk_length`` (e.g.
-            0.5 means that the window shifts of half ``chunk_length``)
+            If the step is not a full ``segment_size`` but less (e.g. a moving window),
+            this indicates the ratio between step step and ``segment_size`` (e.g.
+            0.5 means that the window shifts of half ``segment_size``)
         kwargs : keyword arguments
             These additional keyword arguments, if present, they will be passed
             to ``func``
@@ -1345,14 +1384,14 @@ class Lightcurve(object):
         >>> start, stop, res = lc.analyze_lc_chunks(5, mean_func)
         >>> len(res) == 2
         True
-        >>> np.all(res == 10)
+        >>> np.allclose(res, 10)
         True
         """
-        start, stop = bin_intervals_from_gtis(self.gti, chunk_length,
+        start, stop = bin_intervals_from_gtis(self.gti, segment_size,
                                               self.time,
                                               fraction_step=fraction_step,
                                               dt=self.dt)
-        start_times = self.time[start] - self.dt * 0.5
+        start_times = self.time[start] - 0.5 * self.dt
 
         # Remember that stop is one element above the last element, because
         # it's defined to be used in intervals start:stop
@@ -1388,20 +1427,101 @@ class Lightcurve(object):
         except ImportError:
             raise ImportError("You need to install Lightkurve to use "
                               "the Lightcurve.to_lightkurve() method.")
-        return lk(time=self.time, flux=self.counts, flux_err=self.counts_err)
+        time = Time(self.time / 86400 + self.mjdref, format="mjd", scale="utc")
+        return lk(time=time, flux=self.counts, flux_err=self.counts_err)
 
     @staticmethod
-    def from_lightkurve(lk):
+    def from_lightkurve(lk, skip_checks=True):
         """
         Creates a new `Lightcurve` from a `lightkurve.LightCurve`.
 
         Parameters
         ----------
         lk : `lightkurve.LightCurve`
-            A lightkurve LightCurve object.
+            A lightkurve LightCurve object
+        skip_checks: bool
+            If True, the user specifies that data are already sorted and contain no
+            infinite or nan points. Use at your own risk.
         """
-        return Lightcurve(time=lc.time, counts=lc.flux,
-                          err=lc.flux_err, input_counts=False)
+
+        return Lightcurve(time=lk.time, counts=lk.flux,
+                          err=lk.flux_err, input_counts=False,
+                          skip_checks=skip_checks)
+
+    def to_astropy_timeseries(self):
+        return self._to_astropy_object(kind="timeseries")
+
+    def to_astropy_table(self):
+        return self._to_astropy_object(kind="table")
+
+    def _to_astropy_object(self, kind="table"):
+        data = {}
+
+        for attr in ['_counts', '_counts_err', '_countrate', '_countrate_err',
+                     '_bin_lo', '_bin_hi']:
+            if hasattr(self, attr) and getattr(self, attr) is not None:
+                data[attr.lstrip('_')] = np.asarray(getattr(self, attr))
+
+        if kind.lower() == "table":
+            data['time'] = self.time
+            ts = Table(data)
+        elif kind.lower() == "timeseries":
+            from astropy.timeseries import TimeSeries
+            ts = TimeSeries(data=data, time=TimeDelta(self.time * u.s))
+        else:  # pragma: no cover
+            raise ValueError("Invalid kind (accepted: table or timeseries)")
+
+        for attr in ['_gti', 'mjdref', '_meancounts', '_meancountrate',
+                     'instr', 'mission', 'dt', 'err_dist']:
+            if hasattr(self, attr) and getattr(self, attr) is not None:
+                ts.meta[attr.lstrip('_')] = getattr(self, attr)
+
+        return ts
+
+    @staticmethod
+    def from_astropy_timeseries(ts, **kwargs):
+        return Lightcurve._from_astropy_object(ts, **kwargs)
+
+    @staticmethod
+    def from_astropy_table(ts, **kwargs):
+        return Lightcurve._from_astropy_object(ts, **kwargs)
+
+    @staticmethod
+    def _from_astropy_object(
+            ts, err_dist='poisson', skip_checks=True):
+
+        if hasattr(ts, 'time'):
+            time = ts.time
+        else:
+            time = ts['time']
+
+        kwargs = ts.meta
+        err = None
+        input_counts = True
+
+        if "counts_err" in ts.colnames:
+            err = ts["counts_err"]
+        elif "countrate_err" in ts.colnames:
+            err = ts["countrate_err"]
+
+        if 'counts' in ts.colnames:
+            counts = ts['counts']
+        elif 'countrate' in ts.colnames:
+            counts = ts['countrate']
+            input_counts = False
+        else:
+            raise ValueError('Input timeseries must contain at least a '
+                             '`counts` or a `countrate` column')
+
+        kwargs.update({'time': time, 'counts': counts, 'err': err,
+                       'input_counts': input_counts,
+                       'skip_checks': skip_checks})
+        if 'err_dist' not in kwargs:
+            kwargs['err_dist'] = err_dist
+
+        lc = Lightcurve(**kwargs)
+
+        return lc
 
     def plot(self, witherrors=False, labels=None, axis=None, title=None,
              marker='-', save=False, filename=None):
@@ -1439,10 +1559,6 @@ class Lightcurve(object):
         filename : str
             File name of the image to save. Depends on the boolean ``save``.
         """
-        try:
-            import matplotlib.pyplot as plt
-        except ImportError:
-            raise ImportError("Matplotlib required for plot()")
 
         fig = plt.figure()
         if witherrors:
@@ -1479,106 +1595,65 @@ class Lightcurve(object):
         else:
             plt.show(block=False)
 
-    def write(self, filename, format_='pickle', **kwargs):
+    @classmethod
+    def read(cls, filename, fmt=None, format_=None, err_dist='gauss',
+             skip_checks=False):
         """
-        Write a :class:`Lightcurve` object to file. Currently supported formats are
+        Read a :class:`Lightcurve` object from file.
+
+        Currently supported formats are
 
         * pickle (not recommended for long-term storage)
-        * HDF5
-        * ASCII
+        * hea : FITS Light curves from HEASARC-supported missions.
+        * any other formats compatible with the writers in
+          :class:`astropy.table.Table` (ascii.ecsv, hdf5, etc.)
 
-        Parameters
-        ----------
-        filename: str
-            Path and file name for the output file.
-
-        format\_: str
-            Available options are 'pickle', 'hdf5', 'ascii'
-        """
-        _ = self.counts, self.counts_err, \
-            self.countrate, self.countrate_err, \
-            self.gti
-        if format_ == 'ascii':
-            io.write(np.array([self.time, self.counts, self.counts_err]).T,
-                     filename, format_, fmt=["%s", "%s", "%s"])
-        elif format_ == 'pickle':
-            io.write(self, filename, format_)
-        elif format_ == 'hdf5':
-            io.write(self, filename, format_)
-        else:
-            utils.simon("Format not understood.")
-
-    def read(self, filename, format_='pickle', default_err_dist='gauss'):
-        """
-        Read a :class:`Lightcurve` object from file. Currently supported formats are
-
-        * pickle (not recommended for long-term storage)
-        * HDF5
-        * ASCII
+        Files that need the :class:`astropy.table.Table` interface MUST contain
+        at least a ``time`` column and a ``counts`` or ``countrate`` column.
+        The default ascii format is enhanced CSV (ECSV). Data formats
+        supporting the serialization of metadata (such as ECSV and HDF5) can
+        contain all lightcurve attributes such as ``dt``, ``gti``, etc with
+        no significant loss of information. Other file formats might lose part
+        of the metadata, so must be used with care.
 
         Parameters
         ----------
         filename: str
             Path and file name for the file to be read.
 
-        format\_: str
-            Available options are 'pickle', 'hdf5', 'ascii'
+        fmt: str
+            Available options are 'pickle', 'hea', and any `Table`-supported
+            format such as 'hdf5', 'ascii.ecsv', etc.
 
         Other parameters
         ----------------
 
-        default_err_dist: str, default='gauss'
+        err_dist: str, default='gauss'
             Default error distribution if not specified in the file (e.g. for
             ASCII files). The default is 'gauss' just because it is likely
             that people using ASCII light curves will want to specify Gaussian
             error bars, if any.
+        skip_checks : bool
+            See :class:`Lightcurve` documentation
 
         Returns
         --------
-        lc : ``astropy.table`` or ``dict`` or :class:`Lightcurve` object
-            * If ``format\_`` is ``ascii``: ``astropy.table`` is returned.
-            * If ``format\_`` is ``hdf5``: dictionary with key-value pairs is returned.
-            * If ``format\_`` is ``pickle``: :class:`Lightcurve` object is returned.
+        lc : :class:`Lightcurve` object
         """
+        if fmt is None and format_ is not None:
+            warnings.warn(
+                "The format_ keyword for read and write is deprecated. "
+                "Use fmt instead", DeprecationWarning)
+            fmt = format_
 
-        if format_ == 'ascii':
-            data_raw = io.read(filename, format_,
-                               names=['time', 'counts', 'counts_err'])
+        if fmt.lower() in ('hea', 'ogip'):
+            data = lcurve_from_fits(filename)
+            data.update({'err_dist': err_dist, 'skip_checks': skip_checks})
+            return Lightcurve(**data)
 
-            data = {'time': np.array(data_raw['time']),
-                    'counts': np.array(data_raw['counts']),
-                    'counts_err': np.array(data_raw['counts_err'])}
-            data['dt'] = np.median(np.diff(data['time']))
-            data['gti'] = np.array([[data['time'][0] - data['dt'] / 0,
-                                     data['time'][-1] + data['dt'] / 0]])
-            # We use default_err_dist == 'gauss' just because people using
-            # ASCII files will generally use Gaussian errors. This can be
-            # changed from the command line.
-            data['err_dist'] = default_err_dist
-            data['mjdref'] = 0
-        elif format_ == 'hdf5':
-            data_raw = io.read(filename, format_)
+        return super().read(filename=filename, fmt=fmt)
 
-            data = {'time': np.array(data_raw['_time']),
-                    'counts': np.array(data_raw['_counts']),
-                    'counts_err': np.array(data_raw['_counts_err'])}
-            data['dt'] = data_raw['dt']
-            data['gti'] = None
-            if 'gti' in data_raw:
-                data['gti'] = data_raw['gti']
-            data['err_dist'] = data_raw['err_dist']
-            data['mjdref'] = data_raw['mjdref']
-        elif format_ == 'pickle':
-            return io.read(filename, format_)
-        else:
-            utils.simon("Format not understood.")
-            return None
-        return Lightcurve(data['time'], data['counts'], err=data['counts_err'],
-                          dt=data['dt'], skip_checks=True, gti=data['gti'],
-                          err_dist=data['err_dist'],
-                          mjdref=data['mjdref'])
-
-    def split_by_gti(self, min_points=2):
+    def split_by_gti(self, gti=None, min_points=2):
         """
         Split the current :class:`Lightcurve` object into a list of :class:`Lightcurve` objects, one
         for each continuous GTI segment as defined in the ``gti`` attribute.
@@ -1594,14 +1669,18 @@ class Lightcurve(object):
         list_of_lcs : list
             A list of :class:`Lightcurve` objects, one for each GTI segment
         """
+
+        if gti is None:
+            gti = self.gti
+
         list_of_lcs = []
 
-        start_bins, stop_bins = gti_border_bins(self.gti, self.time, self.dt)
+        start_bins, stop_bins = gti_border_bins(gti, self.time, self.dt)
         for i in range(len(start_bins)):
             start = start_bins[i]
             stop = stop_bins[i]
 
-            if np.isclose(stop-start, 1):
+            if np.isclose(stop - start, 1):
                 logging.warning("Segment with a single time bin! Ignoring this segment!")
                 continue
             if (stop - start) < min_points:
@@ -1610,7 +1689,7 @@ class Lightcurve(object):
             # Note: GTIs are consistent with default in this case!
             new_lc = Lightcurve(self.time[start:stop], self.counts[start:stop],
                                 err=self.counts_err[start:stop],
-                                mjdref=self.mjdref, gti=[self.gti[i]],
+                                mjdref=self.mjdref, gti=[gti[i]],
                                 dt=self.dt, err_dist=self.err_dist,
                                 skip_checks=True)
             list_of_lcs.append(new_lc)
