@@ -1,15 +1,17 @@
-
+import pickle
+from os import error
 import numpy as np
 import numbers
+import warnings
 from scipy import signal
 import astropy.modeling.models
-from stingray.io import write, read
 from stingray import utils
 from stingray import Lightcurve
 from stingray import AveragedPowerspectrum
 
 __all__ = ['Simulator']
 
+
 class Simulator(object):
     """
     Methods to simulate and visualize light curves.
@@ -27,30 +29,46 @@ class Simulator(object):
     rms : float, default 1
         fractional rms of the simulated light curve,
         actual rms is calculated by mean*rms
+    err : float, default 0
+        the errorbars on the final light curve
     red_noise : int, default 1
         multiple of real length of light curve, by
         which to simulate, to avoid red noise leakage
     random_state : int, default None
         seed value for random processes
+    poisson : bool, default False
+        return Poisson-distributed light curves.
     """
 
-    def __init__(self, dt=1, N=1024, mean=0, rms=1, red_noise=1,
-                 random_state=None, tstart=0.0):
+    def __init__(self, dt, N, mean, rms, err=0., red_noise=1,
+                 random_state=None, tstart=0.0, poisson=False):
         self.dt = dt
+
+        if not isinstance(N, (int, np.integer)):
+            raise ValueError("N must be integer!")
+
         self.N = N
+
+        if mean == 0:
+            warnings.warn("Careful! A mean of zero is unphysical!" +
+                          "This may have unintended consequences!")
         self.mean = mean
+        self.nphot = self.mean * self.N
         self.rms = rms
         self.red_noise = red_noise
         self.tstart = tstart
         self.time = dt*np.arange(N) + self.tstart
+        self.nphot_factor = 1000_000
+        self.err = err
+        self.poisson = poisson
 
         # Initialize a tuple of energy ranges with corresponding light curves
         self.channels = []
 
         self.random_state = utils.get_random_state(random_state)
 
-        assert rms<=1, 'Fractional rms must be less than 1.'
-        assert dt>0, 'Time resolution must be greater than 0'
+        assert rms <= 1, 'Fractional rms must be less than 1.'
+        assert dt > 0, 'Time resolution must be greater than 0'
 
     def simulate(self, *args):
         """
@@ -130,7 +148,7 @@ class Simulator(object):
 
         """
         if isinstance(args[0], (numbers.Integral, float)) and len(args) == 1:
-            return  self._simulate_power_law(args[0])
+            return self._simulate_power_law(args[0])
 
         elif isinstance(args[0], astropy.modeling.Model) and len(args) == 1:
             return self._simulate_model(args[0])
@@ -293,9 +311,9 @@ class Simulator(object):
 
         dt = self.dt
 
-        assert t2>t1, 'Secondary peak must be after primary peak.'
-        assert t3>t2, 'End time must be after secondary peak.'
-        assert p2>p1, 'Secondary peak must be greater than primary peak.'
+        assert t2 > t1, 'Secondary peak must be after primary peak.'
+        assert t3 > t2, 'End time must be after secondary peak.'
+        assert p2 > p1, 'Secondary peak must be greater than primary peak.'
 
         # Append zeros before start time
         h_primary = np.append(np.zeros(int(t1/dt)), p1)
@@ -318,6 +336,65 @@ class Simulator(object):
 
         return h
 
+    def _find_inverse(self, real, imaginary):
+        """
+        Forms complex numbers corresponding to real and imaginary
+        parts and finds inverse series.
+
+        Parameters
+        ----------
+        real : numpy.ndarray
+            Co-effients corresponding to real parts of complex numbers
+        imaginary : numpy.ndarray
+            Co-efficients correspondong to imaginary parts of complex
+            numbers
+
+        Returns
+        -------
+        ifft : numpy.ndarray
+            Real inverse fourier transform of complex numbers
+        """
+
+        # Form complex numbers corresponding to each frequency
+        f = [complex(r, i) for r, i in zip(real, imaginary)]
+
+        f = np.hstack([self.mean * self.N * self.red_noise, f])
+
+        # Obtain time series
+        return np.fft.irfft(f, n=self.N * self.red_noise)
+
+    def _timmerkoenig(self, pds_shape):
+        """Straight application of T&K method to a PDS shape.
+
+        """
+        pds_size = pds_shape.size
+
+        real = np.random.normal(size=pds_size) * np.sqrt(0.5 * pds_shape)
+        imaginary = np.random.normal(size=pds_size) * np.sqrt(0.5 * pds_shape)
+        imaginary[-1] = 0
+
+        counts = self._find_inverse(real, imaginary)
+
+        self.std = counts.std()
+
+        rescaled_counts = self._extract_and_scale(counts)
+        err = np.zeros_like(rescaled_counts)
+
+        if self.poisson:
+            bad = rescaled_counts < 0
+            if np.any(bad):
+                warnings.warn("Some bins of the light curve have counts < 0. Setting to 0")
+                rescaled_counts[bad] = 0
+            lc = Lightcurve(self.time, np.random.poisson(rescaled_counts),
+                            err_dist='poisson', dt=self.dt, skip_checks=True)
+            lc.smooth_counts = rescaled_counts
+        else:
+            lc = Lightcurve(self.time, rescaled_counts,
+                            err=err,
+                            err_dist='gauss', dt=self.dt, skip_checks=True)
+
+        return lc
+
     def _simulate_power_law(self, B):
         """
         Generate LightCurve from a power law spectrum.
@@ -334,21 +411,9 @@ class Simulator(object):
         # Define frequencies at which to compute PSD
         w = np.fft.rfftfreq(self.red_noise*self.N, d=self.dt)[1:]
 
-        # Draw two set of 'N' guassian distributed numbers
-        a1 = self.random_state.normal(size=len(w))
-        a2 = self.random_state.normal(size=len(w))
+        pds_shape = np.power((1/w), B)
 
-        # Multiply by (1/w)^B to get real and imaginary parts
-        real = a1 * np.power((1/w),B/2)
-        imaginary = a2 * np.power((1/w),B/2)
-
-        # Obtain time series
-        long_lc = self._find_inverse(real, imaginary)
-        lc = Lightcurve(self.time, self._extract_and_scale(long_lc),
-                        err=np.zeros_like(self.time) + np.sqrt(self.mean),
-                        err_dist='gauss', dt=self.dt)
-
-        return lc
+        return self._timmerkoenig(pds_shape)
 
     def _simulate_power_spectrum(self, s):
         """
@@ -364,24 +429,10 @@ class Simulator(object):
         lightCurve : `LightCurve` object
         """
         # Cast spectrum as numpy array
-        s = np.array(s)
-
-        self.red_noise = 1
-
-        # Draw two set of 'N' guassian distributed numbers
-        a1 = self.random_state.normal(size=len(s))
-        a2 = self.random_state.normal(size=len(s))
-
-        real = a1 * np.sqrt(s)
-        imaginary = a2 * np.sqrt(s)
-
-        lc = self._find_inverse(real, imaginary)
-        lc = Lightcurve(self.time, self._extract_and_scale(lc),
-                        err=np.zeros_like(self.time) + np.sqrt(self.mean),
-                        err_dist='gauss', dt=self.dt)
-
-        return lc
+        pds_shape = np.zeros(s.size * self.red_noise)
+        pds_shape[:s.size] = s
 
+        return self._timmerkoenig(pds_shape)
 
     def _simulate_model(self, model):
         """
@@ -400,23 +451,13 @@ class Simulator(object):
         """
         # Frequencies at which the PSD is to be computed
         # (only positive frequencies, since the signal is real)
-        nbins = self.red_noise*self.N
+        nbins = self.red_noise * self.N
         simfreq = np.fft.rfftfreq(nbins, d=self.dt)[1:]
 
         # Compute PSD from model
         simpsd = model(simfreq)
 
-        fac = np.sqrt(simpsd)
-        pos_real   = self.random_state.normal(size=nbins//2)*fac
-        pos_imag   = self.random_state.normal(size=nbins//2)*fac
-
-        long_lc = self._find_inverse(pos_real, pos_imag)
-
-        lc = Lightcurve(self.time, self._extract_and_scale(long_lc),
-                        err=np.zeros_like(self.time) + np.sqrt(self.mean),
-                        err_dist='gauss', dt=self.dt)
-        return lc
-
+        return self._timmerkoenig(simpsd)
 
     def _simulate_model_string(self, model_str, params):
         """
@@ -439,29 +480,20 @@ class Simulator(object):
         nbins = self.red_noise*self.N
         simfreq = np.fft.rfftfreq(nbins, d=self.dt)[1:]
 
-        if model_str in dir(models):
-            if isinstance(params, dict):
-                model = eval('models.' + model_str + '(**params)')
-                # Compute PSD from model
-                simpsd = model(simfreq)
-            elif isinstance(params, list):
-                simpsd = eval('models.' + model_str + '(simfreq, params)')
-            else:
-                raise ValueError('Params should be list or dictionary!')
-
-            fac = np.sqrt(simpsd)
-            pos_real   = self.random_state.normal(size=nbins//2)*fac
-            pos_imag   = self.random_state.normal(size=nbins//2)*fac
-
-            long_lc = self._find_inverse(pos_real, pos_imag)
-
-            lc = Lightcurve(self.time, self._extract_and_scale(long_lc),
-                            err=np.zeros_like(self.time) + np.sqrt(self.mean),
-                            err_dist='gauss', dt=self.dt)
-            return lc
-        else:
+        if model_str not in dir(models):
             raise ValueError('Model is not defined!')
 
+        if isinstance(params, dict):
+            model = eval('models.' + model_str + '(**params)')
+            # Compute PSD from model
+            simpsd = model(simfreq)
+        elif isinstance(params, list):
+            simpsd = eval('models.' + model_str + '(simfreq, params)')
+        else:
+            raise ValueError('Params should be list or dictionary!')
+
+        return self._timmerkoenig(simpsd)
+
     def _simulate_impulse_response(self, s, h, mode='same'):
         """
         Generate LightCurve from impulse response. To get
@@ -495,39 +527,10 @@ class Simulator(object):
         elif mode == 'filtered':
             lc = lc[(len(h) - 1):-(len(h) - 1)]
 
-        time = self.dt * np.arange(len(lc)) + self.tstart
-        return Lightcurve(time, lc, err_dist='gauss', dt=self.dt,
-                          err=np.zeros_like(self.time) + np.sqrt(self.mean))
-
-    def _find_inverse(self, real, imaginary):
-        """
-        Forms complex numbers corresponding to real and imaginary
-        parts and finds inverse series.
-
-        Parameters
-        ----------
-        real : numpy.ndarray
-            Co-effients corresponding to real parts of complex numbers
-        imaginary : numpy.ndarray
-            Co-efficients correspondong to imaginary parts of complex
-            numbers
-
-        Returns
-        -------
-        ifft : numpy.ndarray
-            Real inverse fourier transform of complex numbers
-        """
-
-        # Form complex numbers corresponding to each frequency
-        f = [complex(r, i) for r,i in zip(real,imaginary)]
-
-        f = np.hstack([self.mean, f])
-
-        # Obtain real valued time series
-        f_conj = np.conjugate(np.array(f))
-
-        # Obtain time series
-        return np.fft.irfft(f_conj, n=self.red_noise*self.N)
+        time = self.dt * np.arange(0.5, len(lc)) + self.tstart
+        err = np.zeros_like(time)
+        return Lightcurve(time, lc, err_dist='gauss', dt=self.dt, err=err,
+                          skip_checks=True)
 
     def _extract_and_scale(self, long_lc):
         """
@@ -557,10 +560,12 @@ class Simulator(object):
                                           self.red_noise*self.N - self.N+1)
             lc = np.take(long_lc, range(extract, extract + self.N))
 
-        avg = np.mean(lc)
-        std = np.std(lc)
+        mean_lc = np.mean(lc)
 
-        return (lc-avg)/std * self.mean * self.rms + self.mean
+        if self.mean == 0:
+            return (lc-mean_lc)/self.std * self.rms
+        else:
+            return (lc-mean_lc)/self.std * self.mean * self.rms + self.mean
 
     def powerspectrum(self, lc, seg_size=None):
         """
@@ -585,41 +590,43 @@ class Simulator(object):
         return AveragedPowerspectrum(lc, seg_size).power
 
     @staticmethod
-    def read(filename, format_='pickle'):
+    def read(filename, fmt='pickle', format_=None):
         """
-        Imports Simulator object.
+        Reads transfer function from a 'pickle' file.
 
-        Parameters
-        ----------
-        filename : str
-            Name of the Simulator object to be read.
-
-        format_ : str
-            Available option is 'pickle.'
+        Parameter
+        ---------
+        fmt : str
+            the format of the file to be retrieved - accepts 'pickle'.
 
         Returns
         -------
-        object : `Simulator` object
+        data : class instance
+            `TransferFunction` object
         """
-        if format_ == 'pickle':
-            data = read(filename, 'pickle')
-            return data
-        else:
-            raise KeyError("Format not supported.")
+        if format_ is not None:
+            fmt = format_
 
-    def write(self, filename, format_='pickle'):
+        if fmt == 'pickle':
+            with open(filename, "rb") as fobj:
+                return pickle.load(fobj)
+
+        else:
+            raise KeyError("Format not understood.")
+
+    def write(self, filename, fmt="pickle", format_=None):
         """
-        Exports Simulator object.
+        Writes a transfer function to 'pickle' file.
 
         Parameters
         ----------
-        filename : str
-            Name of the Simulator object to be created.
-
-        format_ : str
-            Available options are 'pickle' and 'hdf5'.
+        fmt : str
+            the format of the file to be saved - accepts 'pickle'
         """
-        if format_ == 'pickle':
-            write(self, filename)
+        if format_ is not None:
+            fmt = format_
+        if fmt == 'pickle':
+            with open(filename, "wb") as fobj:
+                pickle.dump(self, fobj)
         else:
-            raise KeyError("Format not supported.")
+            raise KeyError("Format not understood.")
